QScrollBar
{
    ;border: 0
    ;margin: 0
    ;background: ${bg}
}
QScrollBar::handle
{
    ;border: 0
    ;margin: 0
    ;min-width: 4px
    ;min-height: 4px
    ;background: ${bg - #666}
}
QScrollBar:horizontal {height:4px}
QScrollBar:vertical {width:4px}

QScrollBar::left-arrow
,QScrollBar::right-arrow
,QScrollBar::up-arrow
,QScrollBar::down-arrow
{
    ;border: 0
    ;margin: 0
    ;width: 0
    ;height: 0
    ;background: ${bg}
}

QScrollBar::add-page
,QScrollBar::sub-page
{
    ;background: ${bg - #222}
}

#item[CopyQ_selected="true"] QScrollBar::add-page
,#item[CopyQ_selected="true"] QScrollBar::sub-page
{
    ;background: ${sel_bg + #666}
}
#item[CopyQ_selected="true"] QScrollBar::handle
{
    ;background: ${sel_bg - #666}
}
