// This file is generated with "update_icon_font.py" from FontAwesome's metadata.

struct Icon {
    unsigned int unicode;
    bool isBrand;
    const char *searchTerms;
};

constexpr Icon iconList[] = {
{0xf641, false, "ad|advertisement|media|newspaper|promotion|publicity"},
{0xf2b9, false, "address book|contact|directory|index|little black book|rolodex"},
{0xf2bb, false, "address card|about|contact|id|identification|postcard|profile"},
{0xf042, false, "adjust|contrast|dark|light|saturation"},
{0xf5d0, false, "air freshener|car|deodorize|fresh|pine|scent"},
{0xf037, false, "align-center|format|middle|paragraph|text"},
{0xf039, false, "align-justify|format|paragraph|text"},
{0xf036, false, "align-left|format|paragraph|text"},
{0xf038, false, "align-right|format|paragraph|text"},
{0xf461, false, "allergies|allergy|freckles|hand|hives|pox|skin|spots"},
{0xf0f9, false, "ambulance|covid-19|emergency|emt|er|help|hospital|support|vehicle"},
{0xf2a3, false, "american sign language interpreting|asl|deaf|finger|hand|interpret|speak"},
{0xf13d, false, "anchor|berth|boat|dock|embed|link|maritime|moor|secure"},
{0xf103, false, "angle double down|arrows|caret|download|expand"},
{0xf100, false, "angle double left|arrows|back|caret|laquo|previous|quote"},
{0xf101, false, "angle double right|arrows|caret|forward|more|next|quote|raquo"},
{0xf102, false, "angle double up|arrows|caret|collapse|upload"},
{0xf107, false, "angle-down|arrow|caret|download|expand"},
{0xf104, false, "angle-left|arrow|back|caret|less|previous"},
{0xf105, false, "angle-right|arrow|care|forward|more|next"},
{0xf106, false, "angle-up|arrow|caret|collapse|upload"},
{0xf556, false, "angry face|disapprove|emoticon|face|mad|upset"},
{0xf644, false, "ankh|amulet|copper|coptic christianity|copts|crux ansata|egypt|venus"},
{0xf5d1, false, "fruit apple|fall|fruit|fuji|macintosh|orchard|seasonal|vegan"},
{0xf187, false, "archive|box|package|save|storage"},
{0xf557, false, "archway|arc|monument|road|street|tunnel"},
{0xf358, false, "alternate arrow circle down|arrow-circle-o-down|download"},
{0xf359, false, "alternate arrow circle left|arrow-circle-o-left|back|previous"},
{0xf35a, false, "alternate arrow circle right|arrow-circle-o-right|forward|next"},
{0xf35b, false, "alternate arrow circle up|arrow-circle-o-up"},
{0xf0ab, false, "arrow circle down|download"},
{0xf0a8, false, "arrow circle left|back|previous"},
{0xf0a9, false, "arrow circle right|forward|next"},
{0xf0aa, false, "arrow circle up|upload"},
{0xf063, false, "arrow-down|download"},
{0xf060, false, "arrow-left|back|previous"},
{0xf061, false, "arrow-right|forward|next"},
{0xf062, false, "arrow-up|forward|upload"},
{0xf0b2, false, "alternate arrows|arrow|arrows|bigger|enlarge|expand|fullscreen|move|position|reorder|resize"},
{0xf337, false, "alternate arrows horizontal|arrows-h|expand|horizontal|landscape|resize|wide"},
{0xf338, false, "alternate arrows vertical|arrows-v|expand|portrait|resize|tall|vertical"},
{0xf2a2, false, "assistive listening systems|amplify|audio|deaf|ear|headset|hearing|sound"},
{0xf069, false, "asterisk|annotation|details|reference|star"},
{0xf1fa, false, "at|address|author|e-mail|email|handle"},
{0xf558, false, "atlas|book|directions|geography|globe|map|travel|wayfinding"},
{0xf5d2, false, "atom|atheism|chemistry|electron|ion|isotope|neutron|nuclear|proton|science"},
{0xf29e, false, "audio description|blind|narration|video|visual"},
{0xf559, false, "award|honor|praise|prize|recognition|ribbon|trophy"},
{0xf77c, false, "baby|child|diaper|doll|human|infant|kid|offspring|person|sprout"},
{0xf77d, false, "baby carriage|buggy|carrier|infant|push|stroller|transportation|walk|wheels"},
{0xf55a, false, "backspace|command|delete|erase|keyboard|undo"},
{0xf04a, false, "backward|previous|rewind"},
{0xf7e5, false, "bacon|blt|breakfast|ham|lard|meat|pancetta|pork|rasher"},
{0xe059, false, "bacteria|antibiotic|antibody|covid-19|health|organism|sick"},
{0xe05a, false, "bacterium|antibiotic|antibody|covid-19|health|organism|sick"},
{0xf666, false, "bahá'í|bahai|bahá'í|star"},
{0xf24e, false, "balance scale|balanced|justice|legal|measure|weight"},
{0xf515, false, "balance scale (left-weighted)|justice|legal|measure|unbalanced|weight"},
{0xf516, false, "balance scale (right-weighted)|justice|legal|measure|unbalanced|weight"},
{0xf05e, false, "ban|abort|ban|block|cancel|delete|hide|prohibit|remove|stop|trash"},
{0xf462, false, "band-aid|bandage|boo boo|first aid|ouch"},
{0xf02a, false, "barcode|info|laser|price|scan|upc"},
{0xf0c9, false, "bars|checklist|drag|hamburger|list|menu|nav|navigation|ol|reorder|settings|todo|ul"},
{0xf433, false, "baseball ball|foul|hardball|league|leather|mlb|softball|sport"},
{0xf434, false, "basketball ball|dribble|dunk|hoop|nba"},
{0xf2cd, false, "bath|clean|shower|tub|wash"},
{0xf244, false, "battery empty|charge|dead|power|status"},
{0xf240, false, "battery full|charge|power|status"},
{0xf242, false, "battery 1/2 full|charge|power|status"},
{0xf243, false, "battery 1/4 full|charge|low|power|status"},
{0xf241, false, "battery 3/4 full|charge|power|status"},
{0xf236, false, "bed|lodging|mattress|rest|sleep|travel"},
{0xf0fc, false, "beer|alcohol|ale|bar|beverage|brewery|drink|lager|liquor|mug|stein"},
{0xf0f3, false, "bell|alarm|alert|chime|notification|reminder"},
{0xf1f6, false, "bell slash|alert|cancel|disabled|notification|off|reminder"},
{0xf55b, false, "bezier curve|curves|illustrator|lines|path|vector"},
{0xf647, false, "bible|book|catholicism|christianity|god|holy"},
{0xf206, false, "bicycle|bike|gears|pedal|transportation|vehicle"},
{0xf84a, false, "biking|bicycle|bike|cycle|cycling|ride|wheel"},
{0xf1e5, false, "binoculars|glasses|magnify|scenic|spyglass|view"},
{0xf780, false, "biohazard|covid-19|danger|dangerous|hazmat|medical|radioactive|toxic|waste|zombie"},
{0xf1fd, false, "birthday cake|anniversary|bakery|candles|celebration|dessert|frosting|holiday|party|pastry"},
{0xf517, false, "blender|cocktail|milkshake|mixer|puree|smoothie"},
{0xf6b6, false, "blender phone|appliance|cocktail|communication|fantasy|milkshake|mixer|puree|silly|smoothie"},
{0xf29d, false, "blind|cane|disability|person|sight"},
{0xf781, false, "blog|journal|log|online|personal|post|web 2.0|wordpress|writing"},
{0xf032, false, "bold|emphasis|format|text"},
{0xf0e7, false, "lightning bolt|electricity|lightning|weather|zap"},
{0xf1e2, false, "bomb|error|explode|fuse|grenade|warning"},
{0xf5d7, false, "bone|calcium|dog|skeletal|skeleton|tibia"},
{0xf55c, false, "bong|aparatus|cannabis|marijuana|pipe|smoke|smoking"},
{0xf02d, false, "book|diary|documentation|journal|library|read"},
{0xf6b7, false, "book of the dead|Dungeons & Dragons|crossbones|d&d|dark arts|death|dnd|documentation|evil|fantasy|halloween|holiday|necronomicon|read|skull|spell"},
{0xf7e6, false, "medical book|diary|documentation|health|history|journal|library|read|record"},
{0xf518, false, "book open|flyer|library|notebook|open book|pamphlet|reading"},
{0xf5da, false, "book reader|flyer|library|notebook|open book|pamphlet|reading"},
{0xf02e, false, "bookmark|favorite|marker|read|remember|save"},
{0xf84c, false, "border all|cell|grid|outline|stroke|table"},
{0xf850, false, "border none|cell|grid|outline|stroke|table"},
{0xf853, false, "border style"},
{0xf436, false, "bowling ball|alley|candlepin|gutter|lane|strike|tenpin"},
{0xf466, false, "box|archive|container|package|storage"},
{0xf49e, false, "box open|archive|container|package|storage|unpack"},
{0xe05b, false, "tissue box|cough|covid-19|kleenex|mucus|nose|sneeze|snot"},
{0xf468, false, "boxes|archives|inventory|storage|warehouse"},
{0xf2a1, false, "braille|alphabet|blind|dots|raised|vision"},
{0xf5dc, false, "brain|cerebellum|gray matter|intellect|medulla oblongata|mind|noodle|wit"},
{0xf7ec, false, "bread slice|bake|bakery|baking|dough|flour|gluten|grain|sandwich|sourdough|toast|wheat|yeast"},
{0xf0b1, false, "briefcase|bag|business|luggage|office|work"},
{0xf469, false, "medical briefcase|doctor|emt|first aid|health"},
{0xf519, false, "broadcast tower|airwaves|antenna|radio|reception|waves"},
{0xf51a, false, "broom|clean|firebolt|fly|halloween|nimbus 2000|quidditch|sweep|witch"},
{0xf55d, false, "brush|art|bristles|color|handle|paint"},
{0xf188, false, "bug|beetle|error|insect|report"},
{0xf1ad, false, "building|apartment|business|city|company|office|work"},
{0xf0a1, false, "bullhorn|announcement|broadcast|louder|megaphone|share"},
{0xf140, false, "bullseye|archery|goal|objective|target"},
{0xf46a, false, "burn|caliente|energy|fire|flame|gas|heat|hot"},
{0xf207, false, "bus|public transportation|transportation|travel|vehicle"},
{0xf55e, false, "bus alt|mta|public transportation|transportation|travel|vehicle"},
{0xf64a, false, "business time|alarm|briefcase|business socks|clock|flight of the conchords|reminder|wednesday"},
{0xf1ec, false, "calculator|abacus|addition|arithmetic|counting|math|multiplication|subtraction"},
{0xf133, false, "calendar|calendar-o|date|event|schedule|time|when"},
{0xf073, false, "alternate calendar|calendar|date|event|schedule|time|when"},
{0xf274, false, "calendar check|accept|agree|appointment|confirm|correct|date|done|event|ok|schedule|select|success|tick|time|todo|when"},
{0xf783, false, "calendar with day focus|date|detail|event|focus|schedule|single day|time|today|when"},
{0xf272, false, "calendar minus|calendar|date|delete|event|negative|remove|schedule|time|when"},
{0xf271, false, "calendar plus|add|calendar|create|date|event|new|positive|schedule|time|when"},
{0xf273, false, "calendar times|archive|calendar|date|delete|event|remove|schedule|time|when|x"},
{0xf784, false, "calendar with week focus|date|detail|event|focus|schedule|single week|time|today|when"},
{0xf030, false, "camera|image|lens|photo|picture|record|shutter|video"},
{0xf083, false, "retro camera|image|lens|photo|picture|record|shutter|video"},
{0xf6bb, false, "campground|camping|fall|outdoors|teepee|tent|tipi"},
{0xf786, false, "candy cane|candy|christmas|holiday|mint|peppermint|striped|xmas"},
{0xf55f, false, "cannabis|bud|chronic|drugs|endica|endo|ganja|marijuana|mary jane|pot|reefer|sativa|spliff|weed|whacky-tabacky"},
{0xf46b, false, "capsules|drugs|medicine|pills|prescription"},
{0xf1b9, false, "car|auto|automobile|sedan|transportation|travel|vehicle"},
{0xf5de, false, "alternate car|auto|automobile|sedan|transportation|travel|vehicle"},
{0xf5df, false, "car battery|auto|electric|mechanic|power"},
{0xf5e1, false, "car crash|accident|auto|automobile|insurance|sedan|transportation|vehicle|wreck"},
{0xf5e4, false, "car side|auto|automobile|sedan|transportation|travel|vehicle"},
{0xf8ff, false, "caravan|camper|motor home|rv|trailer|travel"},
{0xf0d7, false, "caret down|arrow|dropdown|expand|menu|more|triangle"},
{0xf0d9, false, "caret left|arrow|back|previous|triangle"},
{0xf0da, false, "caret right|arrow|forward|next|triangle"},
{0xf150, false, "caret square down|arrow|caret-square-o-down|dropdown|expand|menu|more|triangle"},
{0xf191, false, "caret square left|arrow|back|caret-square-o-left|previous|triangle"},
{0xf152, false, "caret square right|arrow|caret-square-o-right|forward|next|triangle"},
{0xf151, false, "caret square up|arrow|caret-square-o-up|collapse|triangle|upload"},
{0xf0d8, false, "caret up|arrow|collapse|triangle"},
{0xf787, false, "carrot|bugs bunny|orange|vegan|vegetable"},
{0xf218, false, "shopping cart arrow down|download|save|shopping"},
{0xf217, false, "add to shopping cart|add|create|new|positive|shopping"},
{0xf788, false, "cash register|buy|cha-ching|change|checkout|commerce|leaerboard|machine|pay|payment|purchase|store"},
{0xf6be, false, "cat|feline|halloween|holiday|kitten|kitty|meow|pet"},
{0xf0a3, false, "certificate|badge|star|verified"},
{0xf6c0, false, "chair|furniture|seat|sit"},
{0xf51b, false, "chalkboard|blackboard|learning|school|teaching|whiteboard|writing"},
{0xf51c, false, "chalkboard teacher|blackboard|instructor|learning|professor|school|whiteboard|writing"},
{0xf5e7, false, "charging station|electric|ev|tesla|vehicle"},
{0xf1fe, false, "area chart|analytics|area|chart|graph"},
{0xf080, false, "bar chart|analytics|bar|chart|graph"},
{0xf201, false, "line chart|activity|analytics|chart|dashboard|gain|graph|increase|line"},
{0xf200, false, "pie chart|analytics|chart|diagram|graph|pie"},
{0xf00c, false, "check|accept|agree|checkmark|confirm|correct|done|notice|notification|notify|ok|select|success|tick|todo|yes"},
{0xf058, false, "check circle|accept|agree|confirm|correct|done|ok|select|success|tick|todo|yes"},
{0xf560, false, "double check|accept|agree|checkmark|confirm|correct|done|notice|notification|notify|ok|select|success|tick|todo"},
{0xf14a, false, "check square|accept|agree|checkmark|confirm|correct|done|ok|select|success|tick|todo|yes"},
{0xf7ef, false, "cheese|cheddar|curd|gouda|melt|parmesan|sandwich|swiss|wedge"},
{0xf439, false, "chess|board|castle|checkmate|game|king|rook|strategy|tournament"},
{0xf43a, false, "chess bishop|board|checkmate|game|strategy"},
{0xf43c, false, "chess board|board|checkmate|game|strategy"},
{0xf43f, false, "chess king|board|checkmate|game|strategy"},
{0xf441, false, "chess knight|board|checkmate|game|horse|strategy"},
{0xf443, false, "chess pawn|board|checkmate|game|strategy"},
{0xf445, false, "chess queen|board|checkmate|game|strategy"},
{0xf447, false, "chess rook|board|castle|checkmate|game|strategy"},
{0xf13a, false, "chevron circle down|arrow|download|dropdown|menu|more"},
{0xf137, false, "chevron circle left|arrow|back|previous"},
{0xf138, false, "chevron circle right|arrow|forward|next"},
{0xf139, false, "chevron circle up|arrow|collapse|upload"},
{0xf078, false, "chevron-down|arrow|download|expand"},
{0xf053, false, "chevron-left|arrow|back|bracket|previous"},
{0xf054, false, "chevron-right|arrow|bracket|forward|next"},
{0xf077, false, "chevron-up|arrow|collapse|upload"},
{0xf1ae, false, "child|boy|girl|kid|toddler|young"},
{0xf51d, false, "church|building|cathedral|chapel|community|religion"},
{0xf111, false, "circle|circle-thin|diameter|dot|ellipse|notification|round"},
{0xf1ce, false, "circle notched|circle-o-notch|diameter|dot|ellipse|round|spinner"},
{0xf64f, false, "city|buildings|busy|skyscrapers|urban|windows"},
{0xf7f2, false, "medical clinic|covid-19|doctor|general practitioner|hospital|infirmary|medicine|office|outpatient"},
{0xf328, false, "clipboard|copy|notes|paste|record"},
{0xf46c, false, "clipboard with check|accept|agree|confirm|done|ok|select|success|tick|todo|yes"},
{0xf46d, false, "clipboard list|checklist|completed|done|finished|intinerary|ol|schedule|tick|todo|ul"},
{0xf017, false, "clock|date|late|schedule|time|timer|timestamp|watch"},
{0xf24d, false, "clone|arrange|copy|duplicate|paste"},
{0xf20a, false, "closed captioning|cc|deaf|hearing|subtitle|subtitling|text|video"},
{0xf0c2, false, "cloud|atmosphere|fog|overcast|save|upload|weather"},
{0xf381, false, "alternate cloud download|download|export|save"},
{0xf73b, false, "cloud with (a chance of) meatball|FLDSMDFR|food|spaghetti|storm"},
{0xf6c3, false, "cloud with moon|crescent|evening|lunar|night|partly cloudy|sky"},
{0xf73c, false, "cloud with moon and rain|crescent|evening|lunar|night|partly cloudy|precipitation|rain|sky|storm"},
{0xf73d, false, "cloud with rain|precipitation|rain|sky|storm"},
{0xf740, false, "cloud with heavy showers|precipitation|rain|sky|storm"},
{0xf6c4, false, "cloud with sun|clear|day|daytime|fall|outdoors|overcast|partly cloudy"},
{0xf743, false, "cloud with sun and rain|day|overcast|precipitation|storm|summer|sunshower"},
{0xf382, false, "alternate cloud upload|cloud-upload|import|save|upload"},
{0xf561, false, "cocktail|alcohol|beverage|drink|gin|glass|margarita|martini|vodka"},
{0xf121, false, "code|brackets|code|development|html"},
{0xf126, false, "code branch|branch|code-fork|fork|git|github|rebase|svn|vcs|version"},
{0xf0f4, false, "coffee|beverage|breakfast|cafe|drink|fall|morning|mug|seasonal|tea"},
{0xf013, false, "cog|gear|mechanical|settings|sprocket|wheel"},
{0xf085, false, "cogs|gears|mechanical|settings|sprocket|wheel"},
{0xf51e, false, "coins|currency|dime|financial|gold|money|penny"},
{0xf0db, false, "columns|browser|dashboard|organize|panes|split"},
{0xf075, false, "comment|bubble|chat|commenting|conversation|feedback|message|note|notification|sms|speech|texting"},
{0xf27a, false, "alternate comment|bubble|chat|commenting|conversation|feedback|message|note|notification|sms|speech|texting"},
{0xf651, false, "comment dollar|bubble|chat|commenting|conversation|feedback|message|money|note|notification|pay|sms|speech|spend|texting|transfer"},
{0xf4ad, false, "comment dots|bubble|chat|commenting|conversation|feedback|message|more|note|notification|reply|sms|speech|texting"},
{0xf7f5, false, "alternate medical chat|advice|bubble|chat|commenting|conversation|diagnose|feedback|message|note|notification|prescription|sms|speech|texting"},
{0xf4b3, false, "comment slash|bubble|cancel|chat|commenting|conversation|feedback|message|mute|note|notification|quiet|sms|speech|texting"},
{0xf086, false, "comments|bubble|chat|commenting|conversation|feedback|message|note|notification|sms|speech|texting"},
{0xf653, false, "comments dollar|bubble|chat|commenting|conversation|feedback|message|money|note|notification|pay|sms|speech|spend|texting|transfer"},
{0xf51f, false, "compact disc|album|bluray|cd|disc|dvd|media|movie|music|record|video|vinyl"},
{0xf14e, false, "compass|directions|directory|location|menu|navigation|safari|travel"},
{0xf066, false, "compress|collapse|fullscreen|minimize|move|resize|shrink|smaller"},
{0xf422, false, "alternate compress|collapse|fullscreen|minimize|move|resize|shrink|smaller"},
{0xf78c, false, "alternate compress arrows|collapse|fullscreen|minimize|move|resize|shrink|smaller"},
{0xf562, false, "concierge bell|attention|hotel|receptionist|service|support"},
{0xf563, false, "cookie|baked good|chips|chocolate|eat|snack|sweet|treat"},
{0xf564, false, "cookie bite|baked good|bitten|chips|chocolate|eat|snack|sweet|treat"},
{0xf0c5, false, "copy|clone|duplicate|file|files-o|paper|paste"},
{0xf1f9, false, "copyright|brand|mark|register|trademark"},
{0xf4b8, false, "couch|chair|cushion|furniture|relax|sofa"},
{0xf09d, false, "credit card|buy|checkout|credit-card-alt|debit|money|payment|purchase"},
{0xf125, false, "crop|design|frame|mask|resize|shrink"},
{0xf565, false, "alternate crop|design|frame|mask|resize|shrink"},
{0xf654, false, "cross|catholicism|christianity|church|jesus"},
{0xf05b, false, "crosshairs|aim|bullseye|gpd|picker|position"},
{0xf520, false, "crow|bird|bullfrog|fauna|halloween|holiday|toad"},
{0xf521, false, "crown|award|favorite|king|queen|royal|tiara"},
{0xf7f7, false, "crutch|cane|injury|mobility|wheelchair"},
{0xf1b2, false, "cube|3d|block|dice|package|square|tesseract"},
{0xf1b3, false, "cubes|3d|block|dice|package|pyramid|square|stack|tesseract"},
{0xf0c4, false, "cut|clip|scissors|snip"},
{0xf1c0, false, "database|computer|development|directory|memory|storage"},
{0xf2a4, false, "deaf|ear|hearing|sign language"},
{0xf747, false, "democrat|american|democratic party|donkey|election|left|left-wing|liberal|politics|usa"},
{0xf108, false, "desktop|computer|cpu|demo|desktop|device|imac|machine|monitor|pc|screen"},
{0xf655, false, "dharmachakra|buddhism|buddhist|wheel of dharma"},
{0xf470, false, "diagnoses|analyze|detect|diagnosis|examine|medicine"},
{0xf522, false, "dice|chance|gambling|game|roll"},
{0xf6cf, false, "dice d20|Dungeons & Dragons|chance|d&d|dnd|fantasy|gambling|game|roll"},
{0xf6d1, false, "dice d6|Dungeons & Dragons|chance|d&d|dnd|fantasy|gambling|game|roll"},
{0xf523, false, "dice five|chance|gambling|game|roll"},
{0xf524, false, "dice four|chance|gambling|game|roll"},
{0xf525, false, "dice one|chance|gambling|game|roll"},
{0xf526, false, "dice six|chance|gambling|game|roll"},
{0xf527, false, "dice three|chance|gambling|game|roll"},
{0xf528, false, "dice two|chance|gambling|game|roll"},
{0xf566, false, "digital tachograph|data|distance|speed|tachometer"},
{0xf5eb, false, "directions|map|navigation|sign|turn"},
{0xf7fa, false, "disease|bacteria|cancer|covid-19|illness|infection|sickness|virus"},
{0xf529, false, "divide|arithmetic|calculus|division|math"},
{0xf567, false, "dizzy face|dazed|dead|disapprove|emoticon|face"},
{0xf471, false, "dna|double helix|genetic|helix|molecule|protein"},
{0xf6d3, false, "dog|animal|canine|fauna|mammal|pet|pooch|puppy|woof"},
{0xf155, false, "dollar sign|$|cost|dollar-sign|money|price|usd"},
{0xf472, false, "dolly|carry|shipping|transport"},
{0xf474, false, "dolly flatbed|carry|inventory|shipping|transport"},
{0xf4b9, false, "donate|contribute|generosity|gift|give"},
{0xf52a, false, "door closed|enter|exit|locked"},
{0xf52b, false, "door open|enter|exit|welcome"},
{0xf192, false, "dot circle|bullseye|notification|target"},
{0xf4ba, false, "dove|bird|fauna|flying|peace|war"},
{0xf019, false, "download|export|hard drive|save|transfer"},
{0xf568, false, "drafting compass|design|map|mechanical drawing|plot|plotting"},
{0xf6d5, false, "dragon|Dungeons & Dragons|d&d|dnd|fantasy|fire|lizard|serpent"},
{0xf5ee, false, "draw polygon|anchors|lines|object|render|shape"},
{0xf569, false, "drum|instrument|music|percussion|snare|sound"},
{0xf56a, false, "drum steelpan|calypso|instrument|music|percussion|reggae|snare|sound|steel|tropical"},
{0xf6d7, false, "drumstick with bite taken out|bone|chicken|leg|meat|poultry|turkey"},
{0xf44b, false, "dumbbell|exercise|gym|strength|weight|weight-lifting"},
{0xf793, false, "dumpster|alley|bin|commercial|trash|waste"},
{0xf794, false, "dumpster fire|alley|bin|commercial|danger|dangerous|euphemism|flame|heat|hot|trash|waste"},
{0xf6d9, false, "dungeon|Dungeons & Dragons|building|d&d|dnd|door|entrance|fantasy|gate"},
{0xf044, false, "edit|edit|pen|pencil|update|write"},
{0xf7fb, false, "egg|breakfast|chicken|easter|shell|yolk"},
{0xf052, false, "eject|abort|cancel|cd|discharge"},
{0xf141, false, "horizontal ellipsis|dots|drag|kebab|list|menu|nav|navigation|ol|reorder|settings|ul"},
{0xf142, false, "vertical ellipsis|dots|drag|kebab|list|menu|nav|navigation|ol|reorder|settings|ul"},
{0xf0e0, false, "envelope|e-mail|email|letter|mail|message|notification|support"},
{0xf2b6, false, "envelope open|e-mail|email|letter|mail|message|notification|support"},
{0xf658, false, "envelope open-text|e-mail|email|letter|mail|message|notification|support"},
{0xf199, false, "envelope square|e-mail|email|letter|mail|message|notification|support"},
{0xf52c, false, "equals|arithmetic|even|match|math"},
{0xf12d, false, "eraser|art|delete|remove|rubber"},
{0xf796, false, "ethernet|cable|cat 5|cat 6|connection|hardware|internet|network|wired"},
{0xf153, false, "euro sign|currency|dollar|exchange|money"},
{0xf362, false, "alternate exchange|arrow|arrows|exchange|reciprocate|return|swap|transfer"},
{0xf12a, false, "exclamation|alert|danger|error|important|notice|notification|notify|problem|warning"},
{0xf06a, false, "exclamation circle|alert|danger|error|important|notice|notification|notify|problem|warning"},
{0xf071, false, "exclamation triangle|alert|danger|error|important|notice|notification|notify|problem|warning"},
{0xf065, false, "expand|bigger|enlarge|fullscreen|resize"},
{0xf424, false, "alternate expand|arrows|bigger|enlarge|fullscreen|resize"},
{0xf31e, false, "alternate expand arrows|bigger|enlarge|fullscreen|move|resize"},
{0xf35d, false, "alternate external link|external-link|new|open|share"},
{0xf360, false, "alternate external link square|external-link-square|new|open|share"},
{0xf06e, false, "eye|look|optic|see|seen|show|sight|views|visible"},
{0xf1fb, false, "eye dropper|beaker|clone|color|copy|eyedropper|pipette"},
{0xf070, false, "eye slash|blind|hide|show|toggle|unseen|views|visible|visiblity"},
{0xf863, false, "fan|ac|air conditioning|blade|blower|cool|hot"},
{0xf049, false, "fast-backward|beginning|first|previous|rewind|start"},
{0xf050, false, "fast-forward|end|last|next"},
{0xe005, false, "faucet|covid-19|drip|house|hygiene|kitchen|sink|water"},
{0xf1ac, false, "fax|business|communicate|copy|facsimile|send"},
{0xf52d, false, "feather|bird|light|plucked|quill|write"},
{0xf56b, false, "alternate feather|bird|light|plucked|quill|write"},
{0xf182, false, "female|human|person|profile|user|woman"},
{0xf0fb, false, "fighter-jet|airplane|fast|fly|goose|maverick|plane|quick|top gun|transportation|travel"},
{0xf15b, false, "file|document|new|page|pdf|resume"},
{0xf15c, false, "alternate file|document|file-text|invoice|new|page|pdf"},
{0xf1c6, false, "archive file|.zip|bundle|compress|compression|download|zip"},
{0xf1c7, false, "audio file|document|mp3|music|page|play|sound"},
{0xf1c9, false, "code file|css|development|document|html"},
{0xf56c, false, "file contract|agreement|binding|document|legal|signature"},
{0xf6dd, false, "file csv|document|excel|numbers|spreadsheets|table"},
{0xf56d, false, "file download|document|export|save"},
{0xf1c3, false, "excel file|csv|document|numbers|spreadsheets|table"},
{0xf56e, false, "file export|download|save"},
{0xf1c5, false, "image file|document|image|jpg|photo|png"},
{0xf56f, false, "file import|copy|document|send|upload"},
{0xf570, false, "file invoice|account|bill|charge|document|payment|receipt"},
{0xf571, false, "file invoice with us dollar|$|account|bill|charge|document|dollar-sign|money|payment|receipt|usd"},
{0xf477, false, "medical file|document|health|history|prescription|record"},
{0xf478, false, "alternate medical file|document|health|history|prescription|record"},
{0xf1c1, false, "pdf file|acrobat|document|preview|save"},
{0xf1c4, false, "powerpoint file|display|document|keynote|presentation"},
{0xf572, false, "file prescription|document|drugs|medical|medicine|rx"},
{0xf573, false, "file signature|John Hancock|contract|document|name"},
{0xf574, false, "file upload|document|import|page|save"},
{0xf1c8, false, "video file|document|m4v|movie|mp4|play"},
{0xf1c2, false, "word file|document|edit|page|text|writing"},
{0xf575, false, "fill|bucket|color|paint|paint bucket"},
{0xf576, false, "fill drip|bucket|color|drop|paint|paint bucket|spill"},
{0xf008, false, "film|cinema|movie|strip|video"},
{0xf0b0, false, "filter|funnel|options|separate|sort"},
{0xf577, false, "fingerprint|human|id|identification|lock|smudge|touch|unique|unlock"},
{0xf06d, false, "fire|burn|caliente|flame|heat|hot|popular"},
{0xf7e4, false, "alternate fire|burn|caliente|flame|heat|hot|popular"},
{0xf134, false, "fire-extinguisher|burn|caliente|fire fighter|flame|heat|hot|rescue"},
{0xf479, false, "first aid|emergency|emt|health|medical|rescue"},
{0xf578, false, "fish|fauna|gold|seafood|swimming"},
{0xf6de, false, "raised fist|Dungeons & Dragons|d&d|dnd|fantasy|hand|ki|monk|resist|strength|unarmed combat"},
{0xf024, false, "flag|country|notice|notification|notify|pole|report|symbol"},
{0xf11e, false, "flag-checkered|notice|notification|notify|pole|racing|report|symbol"},
{0xf74d, false, "united states of america flag|betsy ross|country|old glory|stars|stripes|symbol"},
{0xf0c3, false, "flask|beaker|experimental|labs|science"},
{0xf579, false, "flushed face|embarrassed|emoticon|face"},
{0xf07b, false, "folder|archive|directory|document|file"},
{0xf65d, false, "folder minus|archive|delete|directory|document|file|negative|remove"},
{0xf07c, false, "folder open|archive|directory|document|empty|file|new"},
{0xf65e, false, "folder plus|add|archive|create|directory|document|file|new|positive"},
{0xf031, false, "font|alphabet|glyph|text|type|typeface"},
{0xf4e6, false, "font awesome full logo"},
{0xf44e, false, "football ball|ball|fall|nfl|pigskin|seasonal"},
{0xf04e, false, "forward|forward|next|skip"},
{0xf52e, false, "frog|amphibian|bullfrog|fauna|hop|kermit|kiss|prince|ribbit|toad|wart"},
{0xf119, false, "frowning face|disapprove|emoticon|face|rating|sad"},
{0xf57a, false, "frowning face with open mouth|disapprove|emoticon|face|rating|sad"},
{0xf662, false, "funnel dollar|filter|money|options|separate|sort"},
{0xf1e3, false, "futbol|ball|football|mls|soccer"},
{0xf11b, false, "gamepad|arcade|controller|d-pad|joystick|video|video game"},
{0xf52f, false, "gas pump|car|fuel|gasoline|petrol"},
{0xf0e3, false, "gavel|hammer|judge|law|lawyer|opinion"},
{0xf3a5, false, "gem|diamond|jewelry|sapphire|stone|treasure"},
{0xf22d, false, "genderless|androgynous|asexual|sexless"},
{0xf6e2, false, "ghost|apparition|blinky|clyde|floating|halloween|holiday|inky|pinky|spirit"},
{0xf06b, false, "gift|christmas|generosity|giving|holiday|party|present|wrapped|xmas"},
{0xf79c, false, "gifts|christmas|generosity|giving|holiday|party|present|wrapped|xmas"},
{0xf79f, false, "glass cheers|alcohol|bar|beverage|celebration|champagne|clink|drink|holiday|new year's eve|party|toast"},
{0xf000, false, "martini glass|alcohol|bar|beverage|drink|liquor"},
{0xf57b, false, "alternate glass martini|alcohol|bar|beverage|drink|liquor"},
{0xf7a0, false, "glass whiskey|alcohol|bar|beverage|bourbon|drink|liquor|neat|rye|scotch|whisky"},
{0xf530, false, "glasses|hipster|nerd|reading|sight|spectacles|vision"},
{0xf0ac, false, "globe|all|coordinates|country|earth|global|gps|language|localize|location|map|online|place|planet|translate|travel|world"},
{0xf57c, false, "globe with africa shown|all|country|earth|global|gps|language|localize|location|map|online|place|planet|translate|travel|world"},
{0xf57d, false, "globe with americas shown|all|country|earth|global|gps|language|localize|location|map|online|place|planet|translate|travel|world"},
{0xf57e, false, "globe with asia shown|all|country|earth|global|gps|language|localize|location|map|online|place|planet|translate|travel|world"},
{0xf7a2, false, "globe with europe shown|all|country|earth|global|gps|language|localize|location|map|online|place|planet|translate|travel|world"},
{0xf450, false, "golf ball|caddy|eagle|putt|tee"},
{0xf664, false, "gopuram|building|entrance|hinduism|temple|tower"},
{0xf19d, false, "graduation cap|ceremony|college|graduate|learning|school|student"},
{0xf531, false, "greater than|arithmetic|compare|math"},
{0xf532, false, "greater than equal to|arithmetic|compare|math"},
{0xf57f, false, "grimacing face|cringe|emoticon|face|teeth"},
{0xf580, false, "grinning face|emoticon|face|laugh|smile"},
{0xf581, false, "alternate grinning face|emoticon|face|laugh|smile"},
{0xf582, false, "grinning face with smiling eyes|emoticon|face|laugh|smile"},
{0xf583, false, "grinning face with sweat|embarass|emoticon|face|smile"},
{0xf584, false, "smiling face with heart-eyes|emoticon|face|love|smile"},
{0xf585, false, "grinning squinting face|emoticon|face|laugh|smile"},
{0xf586, false, "rolling on the floor laughing|emoticon|face|happy|smile"},
{0xf587, false, "star-struck|emoticon|face|star-struck"},
{0xf588, false, "face with tears of joy|LOL|emoticon|face"},
{0xf589, false, "face with tongue|LOL|emoticon|face"},
{0xf58a, false, "squinting face with tongue|LOL|emoticon|face"},
{0xf58b, false, "winking face with tongue|LOL|emoticon|face"},
{0xf58c, false, "grinning winking face|emoticon|face|flirt|laugh|smile"},
{0xf58d, false, "grip horizontal|affordance|drag|drop|grab|handle"},
{0xf7a4, false, "grip lines|affordance|drag|drop|grab|handle"},
{0xf7a5, false, "grip lines vertical|affordance|drag|drop|grab|handle"},
{0xf58e, false, "grip vertical|affordance|drag|drop|grab|handle"},
{0xf7a6, false, "guitar|acoustic|instrument|music|rock|rock and roll|song|strings"},
{0xf0fd, false, "h square|directions|emergency|hospital|hotel|map"},
{0xf805, false, "hamburger|bacon|beef|burger|burger king|cheeseburger|fast food|grill|ground beef|mcdonalds|sandwich"},
{0xf6e3, false, "hammer|admin|fix|repair|settings|tool"},
{0xf665, false, "hamsa|amulet|christianity|islam|jewish|judaism|muslim|protection"},
{0xf4bd, false, "hand holding|carry|lift"},
{0xf4be, false, "hand holding heart|carry|charity|gift|lift|package"},
{0xe05c, false, "hand holding medical cross|care|covid-19|donate|help"},
{0xf4c0, false, "hand holding us dollar|$|carry|dollar sign|donation|giving|lift|money|price"},
{0xf4c1, false, "hand holding water|carry|covid-19|drought|grow|lift"},
{0xf258, false, "lizard (hand)|game|roshambo"},
{0xf806, false, "hand with middle finger raised|flip the bird|gesture|hate|rude"},
{0xf256, false, "paper (hand)|game|halt|roshambo|stop"},
{0xf25b, false, "peace (hand)|rest|truce"},
{0xf0a7, false, "hand pointing down|finger|hand-o-down|point"},
{0xf0a5, false, "hand pointing left|back|finger|hand-o-left|left|point|previous"},
{0xf0a4, false, "hand pointing right|finger|forward|hand-o-right|next|point|right"},
{0xf0a6, false, "hand pointing up|finger|hand-o-up|point"},
{0xf25a, false, "pointer (hand)|arrow|cursor|select"},
{0xf255, false, "rock (hand)|fist|game|roshambo"},
{0xf257, false, "scissors (hand)|cut|game|roshambo"},
{0xe05d, false, "hand sparkles|clean|covid-19|hygiene|magic|soap|wash"},
{0xf259, false, "spock (hand)|live long|prosper|salute|star trek|vulcan"},
{0xf4c2, false, "hands|carry|hold|lift"},
{0xf4c4, false, "helping hands|aid|assistance|handshake|partnership|volunteering"},
{0xe05e, false, "hands wash|covid-19|hygiene|soap|wash"},
{0xf2b5, false, "handshake|agreement|greeting|meeting|partnership"},
{0xe05f, false, "handshake alternate slash|broken|covid-19|social distance"},
{0xe060, false, "handshake slash|broken|covid-19|social distance"},
{0xf6e6, false, "hanukiah|candle|hanukkah|jewish|judaism|light"},
{0xf807, false, "hard hat|construction|hardhat|helmet|safety"},
{0xf292, false, "hashtag|Twitter|instagram|pound|social media|tag"},
{0xf8c0, false, "cowboy hat|buckaroo|horse|jackeroo|john b.|old west|pardner|ranch|rancher|rodeo|western|wrangler"},
{0xf8c1, false, "cowboy hat side|buckaroo|horse|jackeroo|john b.|old west|pardner|ranch|rancher|rodeo|western|wrangler"},
{0xf6e8, false, "wizard's hat|Dungeons & Dragons|accessory|buckle|clothing|d&d|dnd|fantasy|halloween|head|holiday|mage|magic|pointy|witch"},
{0xf0a0, false, "hdd|cpu|hard drive|harddrive|machine|save|storage"},
{0xe061, false, "head side cough|cough|covid-19|germs|lungs|respiratory|sick"},
{0xe062, false, "head side-cough-slash|cough|covid-19|germs|lungs|respiratory|sick"},
{0xe063, false, "head side mask|breath|covid-19|filter|respirator|virus"},
{0xe064, false, "head side virus|cold|covid-19|flu|sick"},
{0xf1dc, false, "heading|format|header|text|title"},
{0xf025, false, "headphones|audio|listen|music|sound|speaker"},
{0xf58f, false, "alternate headphones|audio|listen|music|sound|speaker"},
{0xf590, false, "headset|audio|gamer|gaming|listen|live chat|microphone|shot caller|sound|support|telemarketer"},
{0xf004, false, "heart|favorite|like|love|relationship|valentine"},
{0xf7a9, false, "heart broken|breakup|crushed|dislike|dumped|grief|love|lovesick|relationship|sad"},
{0xf21e, false, "heartbeat|ekg|electrocardiogram|health|lifeline|vital signs"},
{0xf533, false, "helicopter|airwolf|apache|chopper|flight|fly|travel"},
{0xf591, false, "highlighter|edit|marker|sharpie|update|write"},
{0xf6ec, false, "hiking|activity|backpack|fall|fitness|outdoors|person|seasonal|walking"},
{0xf6ed, false, "hippo|animal|fauna|hippopotamus|hungry|mammal"},
{0xf1da, false, "history|Rewind|clock|reverse|time|time machine"},
{0xf453, false, "hockey puck|ice|nhl|sport"},
{0xf7aa, false, "holly berry|catwoman|christmas|decoration|flora|halle|holiday|ororo munroe|plant|storm|xmas"},
{0xf015, false, "home|abode|building|house|main"},
{0xf6f0, false, "horse|equus|fauna|mammmal|mare|neigh|pony"},
{0xf7ab, false, "horse head|equus|fauna|mammmal|mare|neigh|pony"},
{0xf0f8, false, "hospital|building|covid-19|emergency room|medical center"},
{0xf47d, false, "alternate hospital|building|covid-19|emergency room|medical center"},
{0xf47e, false, "hospital symbol|clinic|covid-19|emergency|map"},
{0xf80d, false, "hospital with user|covid-19|doctor|network|patient|primary care"},
{0xf593, false, "hot tub|bath|jacuzzi|massage|sauna|spa"},
{0xf80f, false, "hot dog|bun|chili|frankfurt|frankfurter|kosher|polish|sandwich|sausage|vienna|weiner"},
{0xf594, false, "hotel|building|inn|lodging|motel|resort|travel"},
{0xf254, false, "hourglass|hour|minute|sand|stopwatch|time"},
{0xf253, false, "hourglass end|hour|minute|sand|stopwatch|time"},
{0xf252, false, "hourglass half|hour|minute|sand|stopwatch|time"},
{0xf251, false, "hourglass start|hour|minute|sand|stopwatch|time"},
{0xf6f1, false, "damaged house|building|devastation|disaster|home|insurance"},
{0xe065, false, "house user|covid-19|home|isolation|quarantine"},
{0xf6f2, false, "hryvnia|currency|money|ukraine|ukrainian"},
{0xf246, false, "i beam cursor|editing|i-beam|type|writing"},
{0xf810, false, "ice cream|chocolate|cone|dessert|frozen|scoop|sorbet|vanilla|yogurt"},
{0xf7ad, false, "icicles|cold|frozen|hanging|ice|seasonal|sharp"},
{0xf86d, false, "icons|bolt|emoji|heart|image|music|photo|symbols"},
{0xf2c1, false, "identification badge|address|contact|identification|license|profile"},
{0xf2c2, false, "identification card|contact|demographics|document|identification|issued|profile"},
{0xf47f, false, "alternate identification card|contact|demographics|document|identification|issued|profile"},
{0xf7ae, false, "igloo|dome|dwelling|eskimo|home|house|ice|snow"},
{0xf03e, false, "image|album|landscape|photo|picture"},
{0xf302, false, "images|album|landscape|photo|picture"},
{0xf01c, false, "inbox|archive|desk|email|mail|message"},
{0xf03c, false, "indent|align|justify|paragraph|tab"},
{0xf275, false, "industry|building|factory|industrial|manufacturing|mill|warehouse"},
{0xf534, false, "infinity|eternity|forever|math"},
{0xf129, false, "info|details|help|information|more|support"},
{0xf05a, false, "info circle|details|help|information|more|support"},
{0xf033, false, "italic|edit|emphasis|font|format|text|type"},
{0xf669, false, "jedi|crest|force|sith|skywalker|star wars|yoda"},
{0xf595, false, "joint|blunt|cannabis|doobie|drugs|marijuana|roach|smoke|smoking|spliff"},
{0xf66a, false, "journal of the whills|book|force|jedi|sith|star wars|yoda"},
{0xf66b, false, "kaaba|building|cube|islam|muslim"},
{0xf084, false, "key|lock|password|private|secret|unlock"},
{0xf11c, false, "keyboard|accessory|edit|input|text|type|write"},
{0xf66d, false, "khanda|chakkar|sikh|sikhism|sword"},
{0xf596, false, "kissing face|beso|emoticon|face|love|smooch"},
{0xf597, false, "kissing face with smiling eyes|beso|emoticon|face|love|smooch"},
{0xf598, false, "face blowing a kiss|beso|emoticon|face|love|smooch"},
{0xf535, false, "kiwi bird|bird|fauna|new zealand"},
{0xf66f, false, "landmark|building|historic|memorable|monument|politics"},
{0xf1ab, false, "language|dialect|idiom|localize|speech|translate|vernacular"},
{0xf109, false, "laptop|computer|cpu|dell|demo|device|mac|macbook|machine|pc"},
{0xf5fc, false, "laptop code|computer|cpu|dell|demo|develop|device|mac|macbook|machine|pc"},
{0xe066, false, "laptop house|computer|covid-19|device|office|remote|work from home"},
{0xf812, false, "laptop medical|computer|device|ehr|electronic health records|history"},
{0xf599, false, "grinning face with big eyes|LOL|emoticon|face|laugh|smile"},
{0xf59a, false, "laugh face with beaming eyes|LOL|emoticon|face|happy|smile"},
{0xf59b, false, "laughing squinting face|LOL|emoticon|face|happy|smile"},
{0xf59c, false, "laughing winking face|LOL|emoticon|face|happy|smile"},
{0xf5fd, false, "layer group|arrange|develop|layers|map|stack"},
{0xf06c, false, "leaf|eco|flora|nature|plant|vegan"},
{0xf094, false, "lemon|citrus|lemonade|lime|tart"},
{0xf536, false, "less than|arithmetic|compare|math"},
{0xf537, false, "less than equal to|arithmetic|compare|math"},
{0xf3be, false, "alternate level down|arrow|level-down"},
{0xf3bf, false, "alternate level up|arrow|level-up"},
{0xf1cd, false, "life ring|coast guard|help|overboard|save|support"},
{0xf0eb, false, "lightbulb|energy|idea|inspiration|light"},
{0xf0c1, false, "link|attach|attachment|chain|connect"},
{0xf195, false, "turkish lira sign|currency|money|try|turkish"},
{0xf03a, false, "list|checklist|completed|done|finished|ol|todo|ul"},
{0xf022, false, "alternate list|checklist|completed|done|finished|ol|todo|ul"},
{0xf0cb, false, "list-ol|checklist|completed|done|finished|numbers|ol|todo|ul"},
{0xf0ca, false, "list-ul|checklist|completed|done|finished|ol|todo|ul"},
{0xf124, false, "location-arrow|address|compass|coordinate|direction|gps|map|navigation|place"},
{0xf023, false, "lock|admin|lock|open|password|private|protect|security"},
{0xf3c1, false, "lock open|admin|lock|open|password|private|protect|security"},
{0xf309, false, "alternate long arrow down|download|long-arrow-down"},
{0xf30a, false, "alternate long arrow left|back|long-arrow-left|previous"},
{0xf30b, false, "alternate long arrow right|forward|long-arrow-right|next"},
{0xf30c, false, "alternate long arrow up|long-arrow-up|upload"},
{0xf2a8, false, "low vision|blind|eye|sight"},
{0xf59d, false, "luggage cart|bag|baggage|suitcase|travel"},
{0xf604, false, "lungs|air|breath|covid-19|organ|respiratory"},
{0xe067, false, "lungs virus|breath|covid-19|respiratory|sick"},
{0xf0d0, false, "magic|autocomplete|automatic|mage|magic|spell|wand|witch|wizard"},
{0xf076, false, "magnet|Attract|lodestone|tool"},
{0xf674, false, "mail bulk|archive|envelope|letter|post office|postal|postcard|send|stamp|usps"},
{0xf183, false, "male|human|man|person|profile|user"},
{0xf279, false, "map|address|coordinates|destination|gps|localize|location|map|navigation|paper|pin|place|point of interest|position|route|travel"},
{0xf59f, false, "map marked|address|coordinates|destination|gps|localize|location|map|navigation|paper|pin|place|point of interest|position|route|travel"},
{0xf5a0, false, "alternate map marked|address|coordinates|destination|gps|localize|location|map|navigation|paper|pin|place|point of interest|position|route|travel"},
{0xf041, false, "map-marker|address|coordinates|destination|gps|localize|location|map|navigation|paper|pin|place|point of interest|position|route|travel"},
{0xf3c5, false, "alternate map marker|address|coordinates|destination|gps|localize|location|map|navigation|paper|pin|place|point of interest|position|route|travel"},
{0xf276, false, "map pin|address|agree|coordinates|destination|gps|localize|location|map|marker|navigation|pin|place|position|travel"},
{0xf277, false, "map signs|directions|directory|map|signage|wayfinding"},
{0xf5a1, false, "marker|design|edit|sharpie|update|write"},
{0xf222, false, "mars|male"},
{0xf227, false, "mars double"},
{0xf229, false, "mars stroke"},
{0xf22b, false, "mars stroke horizontal"},
{0xf22a, false, "mars stroke vertical"},
{0xf6fa, false, "mask|carnivale|costume|disguise|halloween|secret|super hero"},
{0xf5a2, false, "medal|award|ribbon|star|trophy"},
{0xf0fa, false, "medkit|first aid|firstaid|health|help|support"},
{0xf11a, false, "neutral face|emoticon|face|neutral|rating"},
{0xf5a4, false, "face without mouth|emoticon|face|neutral|rating"},
{0xf5a5, false, "face with rolling eyes|emoticon|face|neutral|rating"},
{0xf538, false, "memory|DIMM|RAM|hardware|storage|technology"},
{0xf676, false, "menorah|candle|hanukkah|jewish|judaism|light"},
{0xf223, false, "mercury|transgender"},
{0xf753, false, "meteor|armageddon|asteroid|comet|shooting star|space"},
{0xf2db, false, "microchip|cpu|hardware|processor|technology"},
{0xf130, false, "microphone|audio|podcast|record|sing|sound|voice"},
{0xf3c9, false, "alternate microphone|audio|podcast|record|sing|sound|voice"},
{0xf539, false, "alternate microphone slash|audio|disable|mute|podcast|record|sing|sound|voice"},
{0xf131, false, "microphone slash|audio|disable|mute|podcast|record|sing|sound|voice"},
{0xf610, false, "microscope|covid-19|electron|lens|optics|science|shrink"},
{0xf068, false, "minus|collapse|delete|hide|minify|negative|remove|trash"},
{0xf056, false, "minus circle|delete|hide|negative|remove|shape|trash"},
{0xf146, false, "minus square|collapse|delete|hide|minify|negative|remove|shape|trash"},
{0xf7b5, false, "mitten|clothing|cold|glove|hands|knitted|seasonal|warmth"},
{0xf10b, false, "mobile phone|apple|call|cell phone|cellphone|device|iphone|number|screen|telephone"},
{0xf3cd, false, "alternate mobile|apple|call|cell phone|cellphone|device|iphone|number|screen|telephone"},
{0xf0d6, false, "money bill|buy|cash|checkout|money|payment|price|purchase"},
{0xf3d1, false, "alternate money bill|buy|cash|checkout|money|payment|price|purchase"},
{0xf53a, false, "wavy money bill|buy|cash|checkout|money|payment|price|purchase"},
{0xf53b, false, "alternate wavy money bill|buy|cash|checkout|money|payment|price|purchase"},
{0xf53c, false, "money check|bank check|buy|checkout|cheque|money|payment|price|purchase"},
{0xf53d, false, "alternate money check|bank check|buy|checkout|cheque|money|payment|price|purchase"},
{0xf5a6, false, "monument|building|historic|landmark|memorable"},
{0xf186, false, "moon|contrast|crescent|dark|lunar|night"},
{0xf5a7, false, "mortar pestle|crush|culinary|grind|medical|mix|pharmacy|prescription|spices"},
{0xf678, false, "mosque|building|islam|landmark|muslim"},
{0xf21c, false, "motorcycle|bike|machine|transportation|vehicle"},
{0xf6fc, false, "mountain|glacier|hiking|hill|landscape|travel|view"},
{0xf8cc, false, "mouse|click|computer|cursor|input|peripheral"},
{0xf245, false, "mouse pointer|arrow|cursor|select"},
{0xf7b6, false, "mug hot|caliente|cocoa|coffee|cup|drink|holiday|hot chocolate|steam|tea|warmth"},
{0xf001, false, "music|lyrics|melody|note|sing|sound"},
{0xf6ff, false, "wired network|computer|connect|ethernet|internet|intranet"},
{0xf22c, false, "neuter"},
{0xf1ea, false, "newspaper|article|editorial|headline|journal|journalism|news|press"},
{0xf53e, false, "not equal|arithmetic|compare|math"},
{0xf481, false, "medical notes|clipboard|doctor|ehr|health|history|records"},
{0xf247, false, "object group|combine|copy|design|merge|select"},
{0xf248, false, "object ungroup|copy|design|merge|select|separate"},
{0xf613, false, "oil can|auto|crude|gasoline|grease|lubricate|petroleum"},
{0xf679, false, "om|buddhism|hinduism|jainism|mantra"},
{0xf700, false, "otter|animal|badger|fauna|fur|mammal|marten"},
{0xf03b, false, "outdent|align|justify|paragraph|tab"},
{0xf815, false, "pager|beeper|cellphone|communication"},
{0xf1fc, false, "paint brush|acrylic|art|brush|color|fill|paint|pigment|watercolor"},
{0xf5aa, false, "paint roller|acrylic|art|brush|color|fill|paint|pigment|watercolor"},
{0xf53f, false, "palette|acrylic|art|brush|color|fill|paint|pigment|watercolor"},
{0xf482, false, "pallet|archive|box|inventory|shipping|warehouse"},
{0xf1d8, false, "paper plane|air|float|fold|mail|paper|send"},
{0xf0c6, false, "paperclip|attach|attachment|connect|link"},
{0xf4cd, false, "parachute box|aid|assistance|rescue|supplies"},
{0xf1dd, false, "paragraph|edit|format|text|writing"},
{0xf540, false, "parking|auto|car|garage|meter"},
{0xf5ab, false, "passport|document|id|identification|issued|travel"},
{0xf67b, false, "pastafarianism|agnosticism|atheism|flying spaghetti monster|fsm"},
{0xf0ea, false, "paste|clipboard|copy|document|paper"},
{0xf04c, false, "pause|hold|wait"},
{0xf28b, false, "pause circle|hold|wait"},
{0xf1b0, false, "paw|animal|cat|dog|pet|print"},
{0xf67c, false, "peace|serenity|tranquility|truce|war"},
{0xf304, false, "pen|design|edit|update|write"},
{0xf305, false, "alternate pen|design|edit|update|write"},
{0xf5ac, false, "pen fancy|design|edit|fountain pen|update|write"},
{0xf5ad, false, "pen nib|design|edit|fountain pen|update|write"},
{0xf14b, false, "pen square|edit|pencil-square|update|write"},
{0xf303, false, "alternate pencil|design|edit|pencil|update|write"},
{0xf5ae, false, "pencil ruler|design|draft|draw|pencil"},
{0xe068, false, "people arrows|covid-19|personal space|social distance|space|spread|users"},
{0xf4ce, false, "people carry|box|carry|fragile|help|movers|package"},
{0xf816, false, "hot pepper|buffalo wings|capsicum|chili|chilli|habanero|jalapeno|mexican|spicy|tabasco|vegetable"},
{0xf295, false, "percent|discount|fraction|proportion|rate|ratio"},
{0xf541, false, "percentage|discount|fraction|proportion|rate|ratio"},
{0xf756, false, "person entering booth|changing|changing room|election|human|person|vote|voting"},
{0xf095, false, "phone|call|earphone|number|support|telephone|voice"},
{0xf879, false, "alternate phone|call|earphone|number|support|telephone|voice"},
{0xf3dd, false, "phone slash|call|cancel|earphone|mute|number|support|telephone|voice"},
{0xf098, false, "phone square|call|earphone|number|support|telephone|voice"},
{0xf87b, false, "alternate phone square|call|earphone|number|support|telephone|voice"},
{0xf2a0, false, "phone volume|call|earphone|number|sound|support|telephone|voice|volume-control-phone"},
{0xf87c, false, "photo video|av|film|image|library|media"},
{0xf4d3, false, "piggy bank|bank|save|savings"},
{0xf484, false, "pills|drugs|medicine|prescription|tablets"},
{0xf818, false, "pizza slice|cheese|chicago|italian|mozzarella|new york|pepperoni|pie|slice|teenage mutant ninja turtles|tomato"},
{0xf67f, false, "place of worship|building|church|holy|mosque|synagogue"},
{0xf072, false, "plane|airplane|destination|fly|location|mode|travel|trip"},
{0xf5af, false, "plane arrival|airplane|arriving|destination|fly|land|landing|location|mode|travel|trip"},
{0xf5b0, false, "plane departure|airplane|departing|destination|fly|location|mode|take off|taking off|travel|trip"},
{0xe069, false, "plane slash|airplane mode|canceled|covid-19|delayed|grounded|travel"},
{0xf04b, false, "play|audio|music|playing|sound|start|video"},
{0xf144, false, "play circle|audio|music|playing|sound|start|video"},
{0xf1e6, false, "plug|connect|electric|online|power"},
{0xf067, false, "plus|add|create|expand|new|positive|shape"},
{0xf055, false, "plus circle|add|create|expand|new|positive|shape"},
{0xf0fe, false, "plus square|add|create|expand|new|positive|shape"},
{0xf2ce, false, "podcast|audio|broadcast|music|sound"},
{0xf681, false, "poll|results|survey|trend|vote|voting"},
{0xf682, false, "poll h|results|survey|trend|vote|voting"},
{0xf2fe, false, "poo|crap|poop|shit|smile|turd"},
{0xf75a, false, "poo storm|bolt|cloud|euphemism|lightning|mess|poop|shit|turd"},
{0xf619, false, "poop|crap|poop|shit|smile|turd"},
{0xf3e0, false, "portrait|id|image|photo|picture|selfie"},
{0xf154, false, "pound sign|currency|gbp|money"},
{0xf011, false, "power off|cancel|computer|on|reboot|restart"},
{0xf683, false, "pray|kneel|preach|religion|worship"},
{0xf684, false, "praying hands|kneel|preach|religion|worship"},
{0xf5b1, false, "prescription|drugs|medical|medicine|pharmacy|rx"},
{0xf485, false, "prescription bottle|drugs|medical|medicine|pharmacy|rx"},
{0xf486, false, "alternate prescription bottle|drugs|medical|medicine|pharmacy|rx"},
{0xf02f, false, "print|business|copy|document|office|paper"},
{0xf487, false, "procedures|EKG|bed|electrocardiogram|health|hospital|life|patient|vital"},
{0xf542, false, "project diagram|chart|graph|network|pert"},
{0xe06a, false, "pump medical|anti-bacterial|clean|covid-19|disinfect|hygiene|medical grade|sanitizer|soap"},
{0xe06b, false, "pump soap|anti-bacterial|clean|covid-19|disinfect|hygiene|sanitizer|soap"},
{0xf12e, false, "puzzle piece|add-on|addon|game|section"},
{0xf029, false, "qrcode|barcode|info|information|scan"},
{0xf128, false, "question|help|information|support|unknown"},
{0xf059, false, "question circle|help|information|support|unknown"},
{0xf458, false, "quidditch|ball|bludger|broom|golden snitch|harry potter|hogwarts|quaffle|sport|wizard"},
{0xf10d, false, "quote-left|mention|note|phrase|text|type"},
{0xf10e, false, "quote-right|mention|note|phrase|text|type"},
{0xf687, false, "quran|book|islam|muslim|religion"},
{0xf7b9, false, "radiation|danger|dangerous|deadly|hazard|nuclear|radioactive|warning"},
{0xf7ba, false, "alternate radiation|danger|dangerous|deadly|hazard|nuclear|radioactive|warning"},
{0xf75b, false, "rainbow|gold|leprechaun|prism|rain|sky"},
{0xf074, false, "random|arrows|shuffle|sort|swap|switch|transfer"},
{0xf543, false, "receipt|check|invoice|money|pay|table"},
{0xf8d9, false, "record vinyl|LP|album|analog|music|phonograph|sound"},
{0xf1b8, false, "recycle|Waste|compost|garbage|reuse|trash"},
{0xf01e, false, "redo|forward|refresh|reload|repeat"},
{0xf2f9, false, "alternate redo|forward|refresh|reload|repeat"},
{0xf25d, false, "registered trademark|copyright|mark|trademark"},
{0xf87d, false, "remove format|cancel|font|format|remove|style|text"},
{0xf3e5, false, "reply|mail|message|respond"},
{0xf122, false, "reply-all|mail|message|respond"},
{0xf75e, false, "republican|american|conservative|election|elephant|politics|republican party|right|right-wing|usa"},
{0xf7bd, false, "restroom|bathroom|john|loo|potty|washroom|waste|wc"},
{0xf079, false, "retweet|refresh|reload|share|swap"},
{0xf4d6, false, "ribbon|badge|cause|lapel|pin"},
{0xf70b, false, "ring|Dungeons & Dragons|Gollum|band|binding|d&d|dnd|engagement|fantasy|gold|jewelry|marriage|precious"},
{0xf018, false, "road|highway|map|pavement|route|street|travel"},
{0xf544, false, "robot|android|automate|computer|cyborg"},
{0xf135, false, "rocket|aircraft|app|jet|launch|nasa|space"},
{0xf4d7, false, "route|directions|navigation|travel"},
{0xf09e, false, "rss|blog|feed|journal|news|writing"},
{0xf143, false, "rss square|blog|feed|journal|news|writing"},
{0xf158, false, "ruble sign|currency|money|rub"},
{0xf545, false, "ruler|design|draft|length|measure|planning"},
{0xf546, false, "ruler combined|design|draft|length|measure|planning"},
{0xf547, false, "ruler horizontal|design|draft|length|measure|planning"},
{0xf548, false, "ruler vertical|design|draft|length|measure|planning"},
{0xf70c, false, "running|exercise|health|jog|person|run|sport|sprint"},
{0xf156, false, "indian rupee sign|currency|indian|inr|money"},
{0xf5b3, false, "crying face|emoticon|face|tear|tears"},
{0xf5b4, false, "loudly crying face|emoticon|face|tear|tears"},
{0xf7bf, false, "satellite|communications|hardware|orbit|space"},
{0xf7c0, false, "satellite dish|SETI|communications|hardware|receiver|saucer|signal|space"},
{0xf0c7, false, "save|disk|download|floppy|floppy-o"},
{0xf549, false, "school|building|education|learn|student|teacher"},
{0xf54a, false, "screwdriver|admin|fix|mechanic|repair|settings|tool"},
{0xf70e, false, "scroll|Dungeons & Dragons|announcement|d&d|dnd|fantasy|paper|script"},
{0xf7c2, false, "sd card|image|memory|photo|save"},
{0xf002, false, "search|bigger|enlarge|find|magnify|preview|zoom"},
{0xf688, false, "search dollar|bigger|enlarge|find|magnify|money|preview|zoom"},
{0xf689, false, "search location|bigger|enlarge|find|magnify|preview|zoom"},
{0xf010, false, "search minus|minify|negative|smaller|zoom|zoom out"},
{0xf00e, false, "search plus|bigger|enlarge|magnify|positive|zoom|zoom in"},
{0xf4d8, false, "seedling|flora|grow|plant|vegan"},
{0xf233, false, "server|computer|cpu|database|hardware|network"},
{0xf61f, false, "shapes|blocks|build|circle|square|triangle"},
{0xf064, false, "share|forward|save|send|social"},
{0xf1e0, false, "alternate share|forward|save|send|social"},
{0xf1e1, false, "alternate share square|forward|save|send|social"},
{0xf14d, false, "share square|forward|save|send|social"},
{0xf20b, false, "shekel sign|currency|ils|money"},
{0xf3ed, false, "alternate shield|achievement|award|block|defend|security|winner"},
{0xe06c, false, "shield virus|antibodies|barrier|covid-19|health|protect"},
{0xf21a, false, "ship|boat|sea|water"},
{0xf48b, false, "shipping fast|express|fedex|mail|overnight|package|ups"},
{0xf54b, false, "shoe prints|feet|footprints|steps|walk"},
{0xf290, false, "shopping bag|buy|checkout|grocery|payment|purchase"},
{0xf291, false, "shopping basket|buy|checkout|grocery|payment|purchase"},
{0xf07a, false, "shopping-cart|buy|checkout|grocery|payment|purchase"},
{0xf2cc, false, "shower|bath|clean|faucet|water"},
{0xf5b6, false, "shuttle van|airport|machine|public-transportation|transportation|travel|vehicle"},
{0xf4d9, false, "sign|directions|real estate|signage|wayfinding"},
{0xf2f6, false, "alternate sign in|arrow|enter|join|log in|login|sign in|sign up|sign-in|signin|signup"},
{0xf2a7, false, "sign language|Translate|asl|deaf|hands"},
{0xf2f5, false, "alternate sign out|arrow|exit|leave|log out|logout|sign-out"},
{0xf012, false, "signal|bars|graph|online|reception|status"},
{0xf5b7, false, "signature|John Hancock|cursive|name|writing"},
{0xf7c4, false, "sim card|hard drive|hardware|portable|storage|technology|tiny"},
{0xe06d, false, "sink|bathroom|covid-19|faucet|kitchen|wash"},
{0xf0e8, false, "sitemap|directory|hierarchy|ia|information architecture|organization"},
{0xf7c5, false, "skating|activity|figure skating|fitness|ice|person|winter"},
{0xf7c9, false, "skiing|activity|downhill|fast|fitness|olympics|outdoors|person|seasonal|slalom"},
{0xf7ca, false, "skiing nordic|activity|cross country|fitness|outdoors|person|seasonal"},
{0xf54c, false, "skull|bones|skeleton|x-ray|yorick"},
{0xf714, false, "skull & crossbones|Dungeons & Dragons|alert|bones|d&d|danger|dead|deadly|death|dnd|fantasy|halloween|holiday|jolly-roger|pirate|poison|skeleton|warning"},
{0xf715, false, "slash|cancel|close|mute|off|stop|x"},
{0xf7cc, false, "sleigh|christmas|claus|fly|holiday|santa|sled|snow|xmas"},
{0xf1de, false, "horizontal sliders|adjust|settings|sliders|toggle"},
{0xf118, false, "smiling face|approve|emoticon|face|happy|rating|satisfied"},
{0xf5b8, false, "beaming face with smiling eyes|emoticon|face|happy|positive"},
{0xf4da, false, "winking face|emoticon|face|happy|hint|joke"},
{0xf75f, false, "smog|dragon|fog|haze|pollution|smoke|weather"},
{0xf48d, false, "smoking|cancer|cigarette|nicotine|smoking status|tobacco"},
{0xf54d, false, "smoking ban|ban|cancel|no smoking|non-smoking"},
{0xf7cd, false, "sms|chat|conversation|message|mobile|notification|phone|sms|texting"},
{0xf7ce, false, "snowboarding|activity|fitness|olympics|outdoors|person"},
{0xf2dc, false, "snowflake|precipitation|rain|winter"},
{0xf7d0, false, "snowman|decoration|frost|frosty|holiday"},
{0xf7d2, false, "snowplow|clean up|cold|road|storm|winter"},
{0xe06e, false, "soap|bubbles|clean|covid-19|hygiene|wash"},
{0xf696, false, "socks|business socks|business time|clothing|feet|flight of the conchords|wednesday"},
{0xf5ba, false, "solar panel|clean|eco-friendly|energy|green|sun"},
{0xf0dc, false, "sort|filter|order"},
{0xf15d, false, "sort alphabetical down|alphabetical|arrange|filter|order|sort-alpha-asc"},
{0xf881, false, "alternate sort alphabetical down|alphabetical|arrange|filter|order|sort-alpha-asc"},
{0xf15e, false, "sort alphabetical up|alphabetical|arrange|filter|order|sort-alpha-desc"},
{0xf882, false, "alternate sort alphabetical up|alphabetical|arrange|filter|order|sort-alpha-desc"},
{0xf160, false, "sort amount down|arrange|filter|number|order|sort-amount-asc"},
{0xf884, false, "alternate sort amount down|arrange|filter|order|sort-amount-asc"},
{0xf161, false, "sort amount up|arrange|filter|order|sort-amount-desc"},
{0xf885, false, "alternate sort amount up|arrange|filter|order|sort-amount-desc"},
{0xf0dd, false, "sort down (descending)|arrow|descending|filter|order|sort-desc"},
{0xf162, false, "sort numeric down|arrange|filter|numbers|order|sort-numeric-asc"},
{0xf886, false, "alternate sort numeric down|arrange|filter|numbers|order|sort-numeric-asc"},
{0xf163, false, "sort numeric up|arrange|filter|numbers|order|sort-numeric-desc"},
{0xf887, false, "alternate sort numeric up|arrange|filter|numbers|order|sort-numeric-desc"},
{0xf0de, false, "sort up (ascending)|arrow|ascending|filter|order|sort-asc"},
{0xf5bb, false, "spa|flora|massage|mindfulness|plant|wellness"},
{0xf197, false, "space shuttle|astronaut|machine|nasa|rocket|space|transportation"},
{0xf891, false, "spell check|dictionary|edit|editor|grammar|text"},
{0xf717, false, "spider|arachnid|bug|charlotte|crawl|eight|halloween"},
{0xf110, false, "spinner|circle|loading|progress"},
{0xf5bc, false, "splotch|Ink|blob|blotch|glob|stain"},
{0xf5bd, false, "spray can|Paint|aerosol|design|graffiti|tag"},
{0xf0c8, false, "square|block|box|shape"},
{0xf45c, false, "square full|block|box|shape"},
{0xf698, false, "alternate square root|arithmetic|calculus|division|math"},
{0xf5bf, false, "stamp|art|certificate|imprint|rubber|seal"},
{0xf005, false, "star|achievement|award|favorite|important|night|rating|score"},
{0xf699, false, "star and crescent|islam|muslim|religion"},
{0xf089, false, "star-half|achievement|award|rating|score|star-half-empty|star-half-full"},
{0xf5c0, false, "alternate star half|achievement|award|rating|score|star-half-empty|star-half-full"},
{0xf69a, false, "star of david|jewish|judaism|religion"},
{0xf621, false, "star of life|doctor|emt|first aid|health|medical"},
{0xf048, false, "step-backward|beginning|first|previous|rewind|start"},
{0xf051, false, "step-forward|end|last|next"},
{0xf0f1, false, "stethoscope|covid-19|diagnosis|doctor|general practitioner|hospital|infirmary|medicine|office|outpatient"},
{0xf249, false, "sticky note|message|note|paper|reminder|sticker"},
{0xf04d, false, "stop|block|box|square"},
{0xf28d, false, "stop circle|block|box|circle|square"},
{0xf2f2, false, "stopwatch|clock|reminder|time"},
{0xe06f, false, "stopwatch 20|ABCs|countdown|covid-19|happy birthday|i will survive|reminder|seconds|time|timer"},
{0xf54e, false, "store|building|buy|purchase|shopping"},
{0xf54f, false, "alternate store|building|buy|purchase|shopping"},
{0xe070, false, "alternate store slash|building|buy|closed|covid-19|purchase|shopping"},
{0xe071, false, "store slash|building|buy|closed|covid-19|purchase|shopping"},
{0xf550, false, "stream|flow|list|timeline"},
{0xf21d, false, "street view|directions|location|map|navigation"},
{0xf0cc, false, "strikethrough|cancel|edit|font|format|text|type"},
{0xf551, false, "stroopwafel|caramel|cookie|dessert|sweets|waffle"},
{0xf12c, false, "subscript|edit|font|format|text|type"},
{0xf239, false, "subway|machine|railway|train|transportation|vehicle"},
{0xf0f2, false, "suitcase|baggage|luggage|move|suitcase|travel|trip"},
{0xf5c1, false, "suitcase rolling|baggage|luggage|move|suitcase|travel|trip"},
{0xf185, false, "sun|brighten|contrast|day|lighter|sol|solar|star|weather"},
{0xf12b, false, "superscript|edit|exponential|font|format|text|type"},
{0xf5c2, false, "hushed face|emoticon|face|shocked"},
{0xf5c3, false, "swatchbook|Pantone|color|design|hue|palette"},
{0xf5c4, false, "swimmer|athlete|head|man|olympics|person|pool|water"},
{0xf5c5, false, "swimming pool|ladder|recreation|swim|water"},
{0xf69b, false, "synagogue|building|jewish|judaism|religion|star of david|temple"},
{0xf021, false, "sync|exchange|refresh|reload|rotate|swap"},
{0xf2f1, false, "alternate sync|exchange|refresh|reload|rotate|swap"},
{0xf48e, false, "syringe|covid-19|doctor|immunizations|medical|needle"},
{0xf0ce, false, "table|data|excel|spreadsheet"},
{0xf45d, false, "table tennis|ball|paddle|ping pong"},
{0xf10a, false, "tablet|apple|device|ipad|kindle|screen"},
{0xf3fa, false, "alternate tablet|apple|device|ipad|kindle|screen"},
{0xf490, false, "tablets|drugs|medicine|pills|prescription"},
{0xf3fd, false, "alternate tachometer|dashboard|fast|odometer|speed|speedometer"},
{0xf02b, false, "tag|discount|label|price|shopping"},
{0xf02c, false, "tags|discount|label|price|shopping"},
{0xf4db, false, "tape|design|package|sticky"},
{0xf0ae, false, "tasks|checklist|downloading|downloads|loading|progress|project management|settings|to do"},
{0xf1ba, false, "taxi|cab|cabbie|car|car service|lyft|machine|transportation|travel|uber|vehicle"},
{0xf62e, false, "teeth|bite|dental|dentist|gums|mouth|smile|tooth"},
{0xf62f, false, "teeth open|dental|dentist|gums bite|mouth|smile|tooth"},
{0xf769, false, "high temperature|cook|covid-19|mercury|summer|thermometer|warm"},
{0xf76b, false, "low temperature|cold|cool|covid-19|mercury|thermometer|winter"},
{0xf7d7, false, "tenge|currency|kazakhstan|money|price"},
{0xf120, false, "terminal|code|command|console|development|prompt"},
{0xf034, false, "text-height|edit|font|format|text|type"},
{0xf035, false, "text width|edit|font|format|text|type"},
{0xf00a, false, "th|blocks|boxes|grid|squares"},
{0xf009, false, "th-large|blocks|boxes|grid|squares"},
{0xf00b, false, "th-list|checklist|completed|done|finished|ol|todo|ul"},
{0xf630, false, "theater masks|comedy|perform|theatre|tragedy"},
{0xf491, false, "thermometer|covid-19|mercury|status|temperature"},
{0xf2cb, false, "thermometer empty|cold|mercury|status|temperature"},
{0xf2c7, false, "thermometer full|fever|hot|mercury|status|temperature"},
{0xf2c9, false, "thermometer 1/2 full|mercury|status|temperature"},
{0xf2ca, false, "thermometer 1/4 full|mercury|status|temperature"},
{0xf2c8, false, "thermometer 3/4 full|mercury|status|temperature"},
{0xf165, false, "thumbs-down|disagree|disapprove|dislike|hand|social|thumbs-o-down"},
{0xf164, false, "thumbs-up|agree|approve|favorite|hand|like|ok|okay|social|success|thumbs-o-up|yes|you got it dude"},
{0xf08d, false, "thumbtack|coordinates|location|marker|pin|thumb-tack"},
{0xf3ff, false, "alternate ticket|movie|pass|support|ticket"},
{0xf00d, false, "times|close|cross|error|exit|incorrect|notice|notification|notify|problem|wrong|x"},
{0xf057, false, "times circle|close|cross|exit|incorrect|notice|notification|notify|problem|wrong|x"},
{0xf043, false, "tint|color|drop|droplet|raindrop|waterdrop"},
{0xf5c7, false, "tint slash|color|drop|droplet|raindrop|waterdrop"},
{0xf5c8, false, "tired face|angry|emoticon|face|grumpy|upset"},
{0xf204, false, "toggle off|switch"},
{0xf205, false, "toggle on|switch"},
{0xf7d8, false, "toilet|bathroom|flush|john|loo|pee|plumbing|poop|porcelain|potty|restroom|throne|washroom|waste|wc"},
{0xf71e, false, "toilet paper|bathroom|covid-19|halloween|holiday|lavatory|prank|restroom|roll"},
{0xe072, false, "toilet paper slash|bathroom|covid-19|halloween|holiday|lavatory|leaves|prank|restroom|roll|trouble|ut oh"},
{0xf552, false, "toolbox|admin|container|fix|repair|settings|tools"},
{0xf7d9, false, "tools|admin|fix|repair|screwdriver|settings|tools|wrench"},
{0xf5c9, false, "tooth|bicuspid|dental|dentist|molar|mouth|teeth"},
{0xf6a0, false, "torah|book|jewish|judaism|religion|scroll"},
{0xf6a1, false, "torii gate|building|shintoism"},
{0xf722, false, "tractor|agriculture|farm|vehicle"},
{0xf25c, false, "trademark|copyright|register|symbol"},
{0xf637, false, "traffic light|direction|road|signal|travel"},
{0xe041, false, "trailer|carry|haul|moving|travel"},
{0xf238, false, "train|bullet|commute|locomotive|railway|subway"},
{0xf7da, false, "tram|crossing|machine|mountains|seasonal|transportation"},
{0xf224, false, "transgender|intersex"},
{0xf225, false, "alternate transgender|intersex"},
{0xf1f8, false, "trash|delete|garbage|hide|remove"},
{0xf2ed, false, "alternate trash|delete|garbage|hide|remove|trash-o"},
{0xf829, false, "trash restore|back|control z|oops|undo"},
{0xf82a, false, "alternative trash restore|back|control z|oops|undo"},
{0xf1bb, false, "tree|bark|fall|flora|forest|nature|plant|seasonal"},
{0xf091, false, "trophy|achievement|award|cup|game|winner"},
{0xf0d1, false, "truck|cargo|delivery|shipping|vehicle"},
{0xf4de, false, "truck loading|box|cargo|delivery|inventory|moving|rental|vehicle"},
{0xf63b, false, "truck monster|offroad|vehicle|wheel"},
{0xf4df, false, "truck moving|cargo|inventory|rental|vehicle"},
{0xf63c, false, "truck side|cargo|vehicle"},
{0xf553, false, "t-shirt|clothing|fashion|garment|shirt"},
{0xf1e4, false, "tty|communication|deaf|telephone|teletypewriter|text"},
{0xf26c, false, "television|computer|display|monitor|television"},
{0xf0e9, false, "umbrella|protection|rain|storm|wet"},
{0xf5ca, false, "umbrella beach|protection|recreation|sand|shade|summer|sun"},
{0xf0cd, false, "underline|edit|emphasis|format|text|writing"},
{0xf0e2, false, "undo|back|control z|exchange|oops|return|rotate|swap"},
{0xf2ea, false, "alternate undo|back|control z|exchange|oops|return|swap"},
{0xf29a, false, "universal access|accessibility|hearing|person|seeing|visual impairment"},
{0xf19c, false, "university|bank|building|college|higher education - students|institution"},
{0xf127, false, "unlink|attachment|chain|chain-broken|remove"},
{0xf09c, false, "unlock|admin|lock|password|private|protect"},
{0xf13e, false, "alternate unlock|admin|lock|password|private|protect"},
{0xf093, false, "upload|hard drive|import|publish"},
{0xf007, false, "user|account|avatar|head|human|man|person|profile"},
{0xf406, false, "alternate user|account|avatar|head|human|man|person|profile"},
{0xf4fa, false, "alternate user slash|account|avatar|head|human|man|person|profile"},
{0xf4fb, false, "user astronaut|avatar|clothing|cosmonaut|nasa|space|suit"},
{0xf4fc, false, "user check|accept|check|person|verified"},
{0xf2bd, false, "user circle|account|avatar|head|human|man|person|profile"},
{0xf4fd, false, "user clock|alert|person|remind|time"},
{0xf4fe, false, "user cog|admin|cog|person|settings"},
{0xf4ff, false, "user edit|edit|pen|pencil|person|update|write"},
{0xf500, false, "user friends|group|people|person|team|users"},
{0xf501, false, "user graduate|cap|clothing|commencement|gown|graduation|person|student"},
{0xf728, false, "user injured|cast|injury|ouch|patient|person|sling"},
{0xf502, false, "user lock|admin|lock|person|private|unlock"},
{0xf0f0, false, "doctor|covid-19|job|medical|nurse|occupation|physician|profile|surgeon"},
{0xf503, false, "user minus|delete|negative|remove"},
{0xf504, false, "user ninja|assassin|avatar|dangerous|deadly|sneaky"},
{0xf82f, false, "nurse|covid-19|doctor|midwife|practitioner|surgeon"},
{0xf234, false, "user plus|add|avatar|positive|sign up|signup|team"},
{0xf21b, false, "user secret|clothing|coat|hat|incognito|person|privacy|spy|whisper"},
{0xf505, false, "user shield|admin|person|private|protect|safe"},
{0xf506, false, "user slash|ban|delete|remove"},
{0xf507, false, "user tag|avatar|discount|label|person|role|special"},
{0xf508, false, "user tie|avatar|business|clothing|formal|professional|suit"},
{0xf235, false, "remove user|archive|delete|remove|x"},
{0xf0c0, false, "users|friends|group|people|persons|profiles|team"},
{0xf509, false, "users cog|admin|cog|group|person|settings|team"},
{0xe073, false, "users slash|disband|friends|group|people|persons|profiles|separate|team|ungroup"},
{0xf2e5, false, "utensil spoon|cutlery|dining|scoop|silverware|spoon"},
{0xf2e7, false, "utensils|cutlery|dining|dinner|eat|food|fork|knife|restaurant"},
{0xf5cb, false, "vector square|anchors|lines|object|render|shape"},
{0xf221, false, "venus|female"},
{0xf226, false, "venus double|female"},
{0xf228, false, "venus mars|Gender"},
{0xe085, false, "vest|biker|fashion|style"},
{0xe086, false, "vest-patches|biker|fashion|style"},
{0xf492, false, "vial|experiment|lab|sample|science|test|test tube"},
{0xf493, false, "vials|experiment|lab|sample|science|test|test tube"},
{0xf03d, false, "video|camera|film|movie|record|video-camera"},
{0xf4e2, false, "video slash|add|create|film|new|positive|record|video"},
{0xf6a7, false, "vihara|buddhism|buddhist|building|monastery"},
{0xe074, false, "virus|bug|covid-19|flu|health|sick|viral"},
{0xe075, false, "virus slash|bug|covid-19|cure|eliminate|flu|health|sick|viral"},
{0xe076, false, "viruses|bugs|covid-19|flu|health|multiply|sick|spread|viral"},
{0xf897, false, "voicemail|answer|inbox|message|phone"},
{0xf45f, false, "volleyball ball|beach|olympics|sport"},
{0xf027, false, "volume down|audio|lower|music|quieter|sound|speaker"},
{0xf6a9, false, "volume mute|audio|music|quiet|sound|speaker"},
{0xf026, false, "volume off|audio|ban|music|mute|quiet|silent|sound"},
{0xf028, false, "volume up|audio|higher|louder|music|sound|speaker"},
{0xf772, false, "vote yea|accept|cast|election|politics|positive|yes"},
{0xf729, false, "cardboard vr|3d|augment|google|reality|virtual"},
{0xf554, false, "walking|exercise|health|pedometer|person|steps"},
{0xf555, false, "wallet|billfold|cash|currency|money"},
{0xf494, false, "warehouse|building|capacity|garage|inventory|storage"},
{0xf773, false, "water|lake|liquid|ocean|sea|swim|wet"},
{0xf83e, false, "square wave|frequency|pulse|signal"},
{0xf496, false, "weight|health|measurement|scale|weight"},
{0xf5cd, false, "hanging weight|anvil|heavy|measurement"},
{0xf193, false, "wheelchair|accessible|handicap|person"},
{0xf1eb, false, "wifi|connection|hotspot|internet|network|wireless"},
{0xf72e, false, "wind|air|blow|breeze|fall|seasonal|weather"},
{0xf410, false, "window close|browser|cancel|computer|development"},
{0xf2d0, false, "window maximize|browser|computer|development|expand"},
{0xf2d1, false, "window minimize|browser|collapse|computer|development"},
{0xf2d2, false, "window restore|browser|computer|development"},
{0xf72f, false, "wine bottle|alcohol|beverage|cabernet|drink|glass|grapes|merlot|sauvignon"},
{0xf4e3, false, "wine glass|alcohol|beverage|cabernet|drink|grapes|merlot|sauvignon"},
{0xf5ce, false, "alternate wine glas|alcohol|beverage|cabernet|drink|grapes|merlot|sauvignon"},
{0xf159, false, "won sign|currency|krw|money"},
{0xf0ad, false, "wrench|construction|fix|mechanic|plumbing|settings|spanner|tool|update"},
{0xf497, false, "x-ray|health|medical|radiological images|radiology|skeleton"},
{0xf157, false, "yen sign|currency|jpy|money"},
{0xf6ad, false, "yin yang|daoism|opposites|taoism"},
{0xf26e, true, "500px"},
{0xf368, true, "accessible icon|accessibility|handicap|person|wheelchair|wheelchair-alt"},
{0xf369, true, "accusoft"},
{0xf6af, true, "acquisitions incorporated|Dungeons & Dragons|d&d|dnd|fantasy|game|gaming|tabletop"},
{0xf170, true, "app.net"},
{0xf36a, true, "adversal"},
{0xf36b, true, "affiliatetheme"},
{0xf834, true, "airbnb"},
{0xf36c, true, "algolia"},
{0xf642, true, "alipay"},
{0xf270, true, "amazon"},
{0xf42c, true, "amazon pay"},
{0xf36d, true, "amilia"},
{0xf17b, true, "android|robot"},
{0xf209, true, "angellist"},
{0xf36e, true, "angry creative"},
{0xf420, true, "angular"},
{0xf36f, true, "app store"},
{0xf370, true, "ios app store"},
{0xf371, true, "apper systems ab"},
{0xf179, true, "apple|fruit|ios|mac|operating system|os|osx"},
{0xf415, true, "apple pay"},
{0xf77a, true, "artstation"},
{0xf372, true, "asymmetrik, ltd."},
{0xf77b, true, "atlassian"},
{0xf373, true, "audible"},
{0xf41c, true, "autoprefixer"},
{0xf374, true, "avianex"},
{0xf421, true, "aviato"},
{0xf375, true, "amazon web services (aws)"},
{0xf2d5, true, "bandcamp"},
{0xf835, true, "battle.net"},
{0xf1b4, true, "behance"},
{0xf1b5, true, "behance square"},
{0xf378, true, "bimobject"},
{0xf171, true, "bitbucket|atlassian|bitbucket-square|git"},
{0xf379, true, "bitcoin"},
{0xf37a, true, "bity"},
{0xf27e, true, "font awesome black tie"},
{0xf37b, true, "blackberry"},
{0xf37c, true, "blogger"},
{0xf37d, true, "blogger b"},
{0xf293, true, "bluetooth"},
{0xf294, true, "bluetooth"},
{0xf836, true, "bootstrap"},
{0xf15a, true, "btc"},
{0xf837, true, "buffer"},
{0xf37f, true, "büromöbel-experte gmbh & co. kg."},
{0xf8a6, true, "buy n large"},
{0xf20d, true, "buysellads"},
{0xf785, true, "canadian maple leaf|canada|flag|flora|nature|plant"},
{0xf42d, true, "amazon pay credit card"},
{0xf1f3, true, "american express credit card|amex"},
{0xf416, true, "apple pay credit card"},
{0xf24c, true, "diner's club credit card"},
{0xf1f2, true, "discover credit card"},
{0xf24b, true, "jcb credit card"},
{0xf1f1, true, "mastercard credit card"},
{0xf1f4, true, "paypal credit card"},
{0xf1f5, true, "stripe credit card"},
{0xf1f0, true, "visa credit card"},
{0xf380, true, "centercode"},
{0xf789, true, "centos|linux|operating system|os"},
{0xf268, true, "chrome|browser"},
{0xf838, true, "chromecast"},
{0xe07d, true, "cloudflare"},
{0xf383, true, "cloudscale.ch"},
{0xf384, true, "cloudsmith"},
{0xf385, true, "cloudversify"},
{0xf1cb, true, "codepen"},
{0xf284, true, "codie pie"},
{0xf78d, true, "confluence|atlassian"},
{0xf20e, true, "connect develop"},
{0xf26d, true, "contao"},
{0xf89e, true, "cotton bureau|clothing|t-shirts|tshirts"},
{0xf388, true, "cpanel"},
{0xf25e, true, "creative commons"},
{0xf4e7, true, "creative commons attribution"},
{0xf4e8, true, "creative commons noncommercial"},
{0xf4e9, true, "creative commons noncommercial (euro sign)"},
{0xf4ea, true, "creative commons noncommercial (yen sign)"},
{0xf4eb, true, "creative commons no derivative works"},
{0xf4ec, true, "creative commons public domain"},
{0xf4ed, true, "alternate creative commons public domain"},
{0xf4ee, true, "creative commons remix"},
{0xf4ef, true, "creative commons share alike"},
{0xf4f0, true, "creative commons sampling"},
{0xf4f1, true, "creative commons sampling +"},
{0xf4f2, true, "creative commons share"},
{0xf4f3, true, "creative commons cc0"},
{0xf6c9, true, "critical role|Dungeons & Dragons|d&d|dnd|fantasy|game|gaming|tabletop"},
{0xf13c, true, "css 3 logo|code"},
{0xf38b, true, "alternate css3 logo"},
{0xf38c, true, "cuttlefish"},
{0xf38d, true, "dungeons & dragons"},
{0xf6ca, true, "d&d beyond|Dungeons & Dragons|d&d|dnd|fantasy|gaming|tabletop"},
{0xe052, true, "dailymotion"},
{0xf210, true, "dashcube"},
{0xe077, true, "deezer"},
{0xf1a5, true, "delicious"},
{0xf38e, true, "deploy.dog"},
{0xf38f, true, "deskpro"},
{0xf6cc, true, "dev"},
{0xf1bd, true, "deviantart"},
{0xf790, true, "dhl|Dalsey|Hillblom and Lynn|german|package|shipping"},
{0xf791, true, "diaspora"},
{0xf1a6, true, "digg logo"},
{0xf391, true, "digital ocean"},
{0xf392, true, "discord"},
{0xf393, true, "discourse"},
{0xf394, true, "dochub"},
{0xf395, true, "docker"},
{0xf396, true, "draft2digital"},
{0xf17d, true, "dribbble"},
{0xf397, true, "dribbble square"},
{0xf16b, true, "dropbox"},
{0xf1a9, true, "drupal logo"},
{0xf399, true, "dyalog"},
{0xf39a, true, "earlybirds"},
{0xf4f4, true, "ebay"},
{0xf282, true, "edge browser|browser|ie"},
{0xe078, true, "edge legacy browser"},
{0xf430, true, "elementor"},
{0xf5f1, true, "ello"},
{0xf423, true, "ember"},
{0xf1d1, true, "galactic empire"},
{0xf299, true, "envira gallery|leaf"},
{0xf39d, true, "erlang"},
{0xf42e, true, "ethereum"},
{0xf2d7, true, "etsy"},
{0xf839, true, "evernote"},
{0xf23e, true, "expeditedssl"},
{0xf09a, true, "facebook|facebook-official|social network"},
{0xf39e, true, "facebook f|facebook"},
{0xf39f, true, "facebook messenger"},
{0xf082, true, "facebook square|social network"},
{0xf6dc, true, "fantasy flight-games|Dungeons & Dragons|d&d|dnd|fantasy|game|gaming|tabletop"},
{0xf797, true, "fedex|Federal Express|package|shipping"},
{0xf798, true, "fedora|linux|operating system|os"},
{0xf799, true, "figma|app|design|interface"},
{0xf269, true, "firefox|browser"},
{0xe007, true, "firefox browser|browser"},
{0xf2b0, true, "first order"},
{0xf50a, true, "alternate first order"},
{0xf3a1, true, "firstdraft"},
{0xf16e, true, "flickr"},
{0xf44d, true, "flipboard"},
{0xf417, true, "fly"},
{0xf2b4, true, "font awesome|meanpath"},
{0xf35c, true, "alternate font awesome"},
{0xf425, true, "font awesome flag"},
{0xf4e6, true, "font awesome full logo"},
{0xf280, true, "fonticons"},
{0xf3a2, true, "fonticons fi"},
{0xf286, true, "fort awesome|castle"},
{0xf3a3, true, "alternate fort awesome|castle"},
{0xf211, true, "forumbee"},
{0xf180, true, "foursquare"},
{0xf2c5, true, "freecodecamp"},
{0xf3a4, true, "freebsd"},
{0xf50b, true, "fulcrum"},
{0xf50c, true, "galactic republic|politics|star wars"},
{0xf50d, true, "galactic senate|star wars"},
{0xf265, true, "get pocket"},
{0xf260, true, "gg currency"},
{0xf261, true, "gg currency circle"},
{0xf1d3, true, "git"},
{0xf841, true, "git alt"},
{0xf1d2, true, "git square"},
{0xf09b, true, "github|octocat"},
{0xf113, true, "alternate github|octocat"},
{0xf092, true, "github square|octocat"},
{0xf3a6, true, "gitkraken"},
{0xf296, true, "gitlab|Axosoft"},
{0xf426, true, "gitter"},
{0xf2a5, true, "glide"},
{0xf2a6, true, "glide g"},
{0xf3a7, true, "gofore"},
{0xf3a8, true, "goodreads"},
{0xf3a9, true, "goodreads g"},
{0xf1a0, true, "google logo"},
{0xf3aa, true, "google drive"},
{0xe079, true, "google pay"},
{0xf3ab, true, "google play"},
{0xf2b3, true, "google plus|google-plus-circle|google-plus-official"},
{0xf0d5, true, "google plus g|google-plus|social network"},
{0xf0d4, true, "google plus square|social network"},
{0xf1ee, true, "google wallet"},
{0xf184, true, "gratipay (gittip)|favorite|heart|like|love"},
{0xf2d6, true, "grav"},
{0xf3ac, true, "gripfire, inc."},
{0xf3ad, true, "grunt"},
{0xe07e, true, "guilded"},
{0xf3ae, true, "gulp"},
{0xf1d4, true, "hacker news"},
{0xf3af, true, "hacker news square"},
{0xf5f7, true, "hackerrank"},
{0xf452, true, "hips"},
{0xf3b0, true, "hireahelper"},
{0xe07f, true, "hive blockchain network"},
{0xf427, true, "hooli"},
{0xf592, true, "hornbill"},
{0xf3b1, true, "hotjar"},
{0xf27c, true, "houzz"},
{0xf13b, true, "html 5 logo"},
{0xf3b2, true, "hubspot"},
{0xe013, true, "ideal"},
{0xf2d8, true, "imdb"},
{0xe080, true, "innosoft"},
{0xf16d, true, "instagram"},
{0xe055, true, "instagram square"},
{0xe081, true, "instalod"},
{0xf7af, true, "intercom|app|customer|messenger"},
{0xf26b, true, "internet-explorer|browser|ie"},
{0xf7b0, true, "invision|app|design|interface"},
{0xf208, true, "ioxhost"},
{0xf83a, true, "itch.io"},
{0xf3b4, true, "itunes"},
{0xf3b5, true, "itunes note"},
{0xf4e4, true, "java"},
{0xf50e, true, "jedi order|star wars"},
{0xf3b6, true, "jenkis"},
{0xf7b1, true, "jira|atlassian"},
{0xf3b7, true, "joget"},
{0xf1aa, true, "joomla logo"},
{0xf3b8, true, "javascript (js)"},
{0xf3b9, true, "javascript (js) square"},
{0xf1cc, true, "jsfiddle"},
{0xf5fa, true, "kaggle"},
{0xf4f5, true, "keybase"},
{0xf3ba, true, "keycdn"},
{0xf3bb, true, "kickstarter"},
{0xf3bc, true, "kickstarter k"},
{0xf42f, true, "korvue"},
{0xf3bd, true, "laravel"},
{0xf202, true, "last.fm"},
{0xf203, true, "last.fm square"},
{0xf212, true, "leanpub"},
{0xf41d, true, "less"},
{0xf3c0, true, "line"},
{0xf08c, true, "linkedin|linkedin-square"},
{0xf0e1, true, "linkedin in|linkedin"},
{0xf2b8, true, "linode"},
{0xf17c, true, "linux|tux"},
{0xf3c3, true, "lyft"},
{0xf3c4, true, "magento"},
{0xf59e, true, "mailchimp"},
{0xf50f, true, "mandalorian"},
{0xf60f, true, "markdown"},
{0xf4f6, true, "mastodon"},
{0xf136, true, "maxcdn"},
{0xf8ca, true, "material design for bootstrap"},
{0xf3c6, true, "medapps"},
{0xf23a, true, "medium"},
{0xf3c7, true, "medium m"},
{0xf3c8, true, "mrt"},
{0xf2e0, true, "meetup"},
{0xf5a3, true, "megaport"},
{0xf7b3, true, "mendeley"},
{0xe01a, true, "micro.blog"},
{0xf3ca, true, "microsoft"},
{0xf3cb, true, "mix"},
{0xf289, true, "mixcloud"},
{0xe056, true, "mixer"},
{0xf3cc, true, "mizuni"},
{0xf285, true, "modx"},
{0xf3d0, true, "monero"},
{0xf3d2, true, "napster"},
{0xf612, true, "neos"},
{0xf5a8, true, "nimblr"},
{0xf419, true, "node.js"},
{0xf3d3, true, "node.js js"},
{0xf3d4, true, "npm"},
{0xf3d5, true, "ns8"},
{0xf3d6, true, "nutritionix"},
{0xe082, true, "octopus deploy"},
{0xf263, true, "odnoklassniki"},
{0xf264, true, "odnoklassniki square"},
{0xf510, true, "old republic|politics|star wars"},
{0xf23d, true, "opencart"},
{0xf19b, true, "openid"},
{0xf26a, true, "opera"},
{0xf23c, true, "optin monster"},
{0xf8d2, true, "orcid"},
{0xf41a, true, "open source initiative"},
{0xf3d7, true, "page4 corporation"},
{0xf18c, true, "pagelines|eco|flora|leaf|leaves|nature|plant|tree"},
{0xf3d8, true, "palfed"},
{0xf3d9, true, "patreon"},
{0xf1ed, true, "paypal"},
{0xf704, true, "penny arcade|Dungeons & Dragons|d&d|dnd|fantasy|game|gaming|pax|tabletop"},
{0xe083, true, "perbyte"},
{0xf3da, true, "periscope"},
{0xf3db, true, "phabricator"},
{0xf3dc, true, "phoenix framework"},
{0xf511, true, "phoenix squadron"},
{0xf457, true, "php"},
{0xf2ae, true, "pied piper logo"},
{0xf1a8, true, "alternate pied piper logo (old)"},
{0xf4e5, true, "pied piper hat (old)|clothing"},
{0xf1a7, true, "pied piper pp logo (old)"},
{0xe01e, true, "pied piper square logo (old)"},
{0xf0d2, true, "pinterest"},
{0xf231, true, "pinterest p"},
{0xf0d3, true, "pinterest square"},
{0xf3df, true, "playstation"},
{0xf288, true, "product hunt"},
{0xf3e1, true, "pushed"},
{0xf3e2, true, "python"},
{0xf1d6, true, "qq"},
{0xf459, true, "quinscape"},
{0xf2c4, true, "quora"},
{0xf4f7, true, "r project"},
{0xf7bb, true, "raspberry pi"},
{0xf2d9, true, "ravelry"},
{0xf41b, true, "react"},
{0xf75d, true, "reacteurope"},
{0xf4d5, true, "readme"},
{0xf1d0, true, "rebel alliance"},
{0xf3e3, true, "red river"},
{0xf1a1, true, "reddit logo"},
{0xf281, true, "reddit alien"},
{0xf1a2, true, "reddit square"},
{0xf7bc, true, "redhat|linux|operating system|os"},
{0xf18b, true, "renren"},
{0xf3e6, true, "replyd"},
{0xf4f8, true, "researchgate"},
{0xf3e7, true, "resolving"},
{0xf5b2, true, "rev.io"},
{0xf3e8, true, "rocket.chat"},
{0xf3e9, true, "rockrms"},
{0xe07a, true, "rust"},
{0xf267, true, "safari|browser"},
{0xf83b, true, "salesforce"},
{0xf41e, true, "sass"},
{0xf3ea, true, "schlix"},
{0xf28a, true, "scribd"},
{0xf3eb, true, "searchengin"},
{0xf2da, true, "sellcast|eercast"},
{0xf213, true, "sellsy"},
{0xf3ec, true, "servicestack"},
{0xf214, true, "shirts in bulk"},
{0xe057, true, "shopify"},
{0xf5b5, true, "shopware"},
{0xf215, true, "simplybuilt"},
{0xf3ee, true, "sistrix"},
{0xf512, true, "sith"},
{0xf7c6, true, "sketch|app|design|interface"},
{0xf216, true, "skyatlas"},
{0xf17e, true, "skype"},
{0xf198, true, "slack logo|anchor|hash|hashtag"},
{0xf3ef, true, "slack hashtag|anchor|hash|hashtag"},
{0xf1e7, true, "slideshare"},
{0xf2ab, true, "snapchat"},
{0xf2ac, true, "snapchat ghost"},
{0xf2ad, true, "snapchat square"},
{0xf1be, true, "soundcloud"},
{0xf7d3, true, "sourcetree"},
{0xf3f3, true, "speakap"},
{0xf83c, true, "speaker deck"},
{0xf1bc, true, "spotify"},
{0xf5be, true, "squarespace"},
{0xf18d, true, "stack exchange"},
{0xf16c, true, "stack overflow"},
{0xf842, true, "stackpath"},
{0xf3f5, true, "staylinked"},
{0xf1b6, true, "steam"},
{0xf1b7, true, "steam square"},
{0xf3f6, true, "steam symbol"},
{0xf3f7, true, "sticker mule"},
{0xf428, true, "strava"},
{0xf429, true, "stripe"},
{0xf42a, true, "stripe s"},
{0xf3f8, true, "studio vinari"},
{0xf1a4, true, "stumbleupon logo"},
{0xf1a3, true, "stumbleupon circle"},
{0xf2dd, true, "superpowers"},
{0xf3f9, true, "supple"},
{0xf7d6, true, "suse|linux|operating system|os"},
{0xf8e1, true, "swift"},
{0xf83d, true, "symfony"},
{0xf4f9, true, "teamspeak"},
{0xf2c6, true, "telegram"},
{0xf3fe, true, "telegram plane"},
{0xf1d5, true, "tencent weibo"},
{0xf69d, true, "the red yeti"},
{0xf5c6, true, "themeco"},
{0xf2b2, true, "themeisle"},
{0xf731, true, "think peaks"},
{0xe07b, true, "tiktok"},
{0xf513, true, "trade federation"},
{0xf181, true, "trello|atlassian"},
{0xf262, true, "tripadvisor"},
{0xf173, true, "tumblr"},
{0xf174, true, "tumblr square"},
{0xf1e8, true, "twitch"},
{0xf099, true, "twitter|social network|tweet"},
{0xf081, true, "twitter square|social network|tweet"},
{0xf42b, true, "typo3"},
{0xf402, true, "uber"},
{0xf7df, true, "ubuntu|linux|operating system|os"},
{0xf403, true, "uikit"},
{0xf8e8, true, "umbraco"},
{0xe084, true, "uncharted software"},
{0xf404, true, "uniregistry"},
{0xe049, true, "unity 3d"},
{0xe07c, true, "unsplash"},
{0xf405, true, "untappd"},
{0xf7e0, true, "ups|United Parcel Service|package|shipping"},
{0xf287, true, "usb"},
{0xf7e1, true, "united states postal service|american|package|shipping|usa"},
{0xf407, true, "us-sunnah foundation"},
{0xf408, true, "vaadin"},
{0xf237, true, "viacoin"},
{0xf2a9, true, "video"},
{0xf2aa, true, "video square"},
{0xf409, true, "viber"},
{0xf40a, true, "vimeo"},
{0xf194, true, "vimeo square"},
{0xf27d, true, "vimeo|vimeo"},
{0xf1ca, true, "vine"},
{0xf189, true, "vk"},
{0xf40b, true, "vnv"},
{0xf41f, true, "vue.js"},
{0xe087, true, "watchman monitoring"},
{0xf83f, true, "waze"},
{0xf5cc, true, "weebly"},
{0xf18a, true, "weibo"},
{0xf1d7, true, "weixin (wechat)"},
{0xf232, true, "what's app"},
{0xf40c, true, "what's app square"},
{0xf40d, true, "whmcs"},
{0xf266, true, "wikipedia w"},
{0xf17a, true, "windows|microsoft|operating system|os"},
{0xf5cf, true, "wix"},
{0xf730, true, "wizards of the coast|Dungeons & Dragons|d&d|dnd|fantasy|game|gaming|tabletop"},
{0xe088, true, "wodu"},
{0xf514, true, "wolf pack battalion"},
{0xf19a, true, "wordpress logo"},
{0xf411, true, "wordpress simple"},
{0xf297, true, "wpbeginner"},
{0xf2de, true, "wpexplorer"},
{0xf298, true, "wpforms"},
{0xf3e4, true, "wpressr|rendact"},
{0xf412, true, "xbox"},
{0xf168, true, "xing"},
{0xf169, true, "xing square"},
{0xf23b, true, "y combinator"},
{0xf19e, true, "yahoo logo"},
{0xf840, true, "yammer"},
{0xf413, true, "yandex"},
{0xf414, true, "yandex international"},
{0xf7e3, true, "yarn"},
{0xf1e9, true, "yelp"},
{0xf2b1, true, "yoast"},
{0xf167, true, "youtube|film|video|youtube-play|youtube-square"},
{0xf431, true, "youtube square"},
{0xf63f, true, "zhihu"}
};
