/*
    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>

    This file is part of CopyQ.

    CopyQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CopyQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CopyQ.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mimetypes.h"

const char mimeText[] = "text/plain";
const char mimeHtml[] = "text/html";
const char mimeUriList[] = "text/uri-list";
const char mimeWindowTitle[] = COPYQ_MIME_PREFIX "owner-window-title";
const char mimeItems[] = COPYQ_MIME_PREFIX "item";
const char mimeItemNotes[] = COPYQ_MIME_PREFIX "item-notes";
const char mimeIcon[] = COPYQ_MIME_PREFIX "item-icon";
const char mimeOwner[] = COPYQ_MIME_PREFIX "owner";
const char mimeClipboardMode[] = COPYQ_MIME_PREFIX "clipboard-mode";
const char mimeCurrentTab[] = COPYQ_MIME_PREFIX "current-tab";
const char mimeSelectedItems[] = COPYQ_MIME_PREFIX "selected-items";
const char mimeCurrentItem[] = COPYQ_MIME_PREFIX "current-item";
const char mimeHidden[] = COPYQ_MIME_PREFIX "hidden";
const char mimeShortcut[] = COPYQ_MIME_PREFIX "shortcut";
const char mimeColor[] = COPYQ_MIME_PREFIX "color";
const char mimeOutputTab[] = COPYQ_MIME_PREFIX "output-tab";
