// This file is generated with "update_icon_font.py" from FontAwesome's metadata.

#ifndef ICONS_H
#define ICONS_H

enum IconId {
    Icon500Px = 0xf26e,
    IconAccessibleIcon = 0xf368,
    IconAccusoft = 0xf369,
    IconAcquisitionsIncorporated = 0xf6af,
    IconAd = 0xf641,
    IconAddressBook = 0xf2b9,
    IconAddressCard = 0xf2bb,
    IconAdjust = 0xf042,
    IconAdn = 0xf170,
    IconAdversal = 0xf36a,
    IconAffiliatetheme = 0xf36b,
    IconAirFreshener = 0xf5d0,
    IconAirbnb = 0xf834,
    IconAlgolia = 0xf36c,
    IconAlignCenter = 0xf037,
    IconAlignJustify = 0xf039,
    IconAlignLeft = 0xf036,
    IconAlignRight = 0xf038,
    IconAlipay = 0xf642,
    IconAllergies = 0xf461,
    IconAmazon = 0xf270,
    IconAmazonPay = 0xf42c,
    IconAmbulance = 0xf0f9,
    IconAmericanSignLanguageInterpreting = 0xf2a3,
    IconAmilia = 0xf36d,
    IconAnchor = 0xf13d,
    IconAndroid = 0xf17b,
    IconAngellist = 0xf209,
    IconAngleDoubleDown = 0xf103,
    IconAngleDoubleLeft = 0xf100,
    IconAngleDoubleRight = 0xf101,
    IconAngleDoubleUp = 0xf102,
    IconAngleDown = 0xf107,
    IconAngleLeft = 0xf104,
    IconAngleRight = 0xf105,
    IconAngleUp = 0xf106,
    IconAngry = 0xf556,
    IconAngrycreative = 0xf36e,
    IconAngular = 0xf420,
    IconAnkh = 0xf644,
    IconAppStore = 0xf36f,
    IconAppStoreIos = 0xf370,
    IconApper = 0xf371,
    IconApple = 0xf179,
    IconAppleAlt = 0xf5d1,
    IconApplePay = 0xf415,
    IconArchive = 0xf187,
    IconArchway = 0xf557,
    IconArrowAltCircleDown = 0xf358,
    IconArrowAltCircleLeft = 0xf359,
    IconArrowAltCircleRight = 0xf35a,
    IconArrowAltCircleUp = 0xf35b,
    IconArrowCircleDown = 0xf0ab,
    IconArrowCircleLeft = 0xf0a8,
    IconArrowCircleRight = 0xf0a9,
    IconArrowCircleUp = 0xf0aa,
    IconArrowDown = 0xf063,
    IconArrowLeft = 0xf060,
    IconArrowRight = 0xf061,
    IconArrowUp = 0xf062,
    IconArrowsAlt = 0xf0b2,
    IconArrowsAltH = 0xf337,
    IconArrowsAltV = 0xf338,
    IconArtstation = 0xf77a,
    IconAssistiveListeningSystems = 0xf2a2,
    IconAsterisk = 0xf069,
    IconAsymmetrik = 0xf372,
    IconAt = 0xf1fa,
    IconAtlas = 0xf558,
    IconAtlassian = 0xf77b,
    IconAtom = 0xf5d2,
    IconAudible = 0xf373,
    IconAudioDescription = 0xf29e,
    IconAutoprefixer = 0xf41c,
    IconAvianex = 0xf374,
    IconAviato = 0xf421,
    IconAward = 0xf559,
    IconAws = 0xf375,
    IconBaby = 0xf77c,
    IconBabyCarriage = 0xf77d,
    IconBackspace = 0xf55a,
    IconBackward = 0xf04a,
    IconBacon = 0xf7e5,
    IconBacteria = 0xe059,
    IconBacterium = 0xe05a,
    IconBahai = 0xf666,
    IconBalanceScale = 0xf24e,
    IconBalanceScaleLeft = 0xf515,
    IconBalanceScaleRight = 0xf516,
    IconBan = 0xf05e,
    IconBandAid = 0xf462,
    IconBandcamp = 0xf2d5,
    IconBarcode = 0xf02a,
    IconBars = 0xf0c9,
    IconBaseballBall = 0xf433,
    IconBasketballBall = 0xf434,
    IconBath = 0xf2cd,
    IconBatteryEmpty = 0xf244,
    IconBatteryFull = 0xf240,
    IconBatteryHalf = 0xf242,
    IconBatteryQuarter = 0xf243,
    IconBatteryThreeQuarters = 0xf241,
    IconBattleNet = 0xf835,
    IconBed = 0xf236,
    IconBeer = 0xf0fc,
    IconBehance = 0xf1b4,
    IconBehanceSquare = 0xf1b5,
    IconBell = 0xf0f3,
    IconBellSlash = 0xf1f6,
    IconBezierCurve = 0xf55b,
    IconBible = 0xf647,
    IconBicycle = 0xf206,
    IconBiking = 0xf84a,
    IconBimobject = 0xf378,
    IconBinoculars = 0xf1e5,
    IconBiohazard = 0xf780,
    IconBirthdayCake = 0xf1fd,
    IconBitbucket = 0xf171,
    IconBitcoin = 0xf379,
    IconBity = 0xf37a,
    IconBlackTie = 0xf27e,
    IconBlackberry = 0xf37b,
    IconBlender = 0xf517,
    IconBlenderPhone = 0xf6b6,
    IconBlind = 0xf29d,
    IconBlog = 0xf781,
    IconBlogger = 0xf37c,
    IconBloggerB = 0xf37d,
    IconBluetooth = 0xf293,
    IconBluetoothB = 0xf294,
    IconBold = 0xf032,
    IconBolt = 0xf0e7,
    IconBomb = 0xf1e2,
    IconBone = 0xf5d7,
    IconBong = 0xf55c,
    IconBook = 0xf02d,
    IconBookDead = 0xf6b7,
    IconBookMedical = 0xf7e6,
    IconBookOpen = 0xf518,
    IconBookReader = 0xf5da,
    IconBookmark = 0xf02e,
    IconBootstrap = 0xf836,
    IconBorderAll = 0xf84c,
    IconBorderNone = 0xf850,
    IconBorderStyle = 0xf853,
    IconBowlingBall = 0xf436,
    IconBox = 0xf466,
    IconBoxOpen = 0xf49e,
    IconBoxTissue = 0xe05b,
    IconBoxes = 0xf468,
    IconBraille = 0xf2a1,
    IconBrain = 0xf5dc,
    IconBreadSlice = 0xf7ec,
    IconBriefcase = 0xf0b1,
    IconBriefcaseMedical = 0xf469,
    IconBroadcastTower = 0xf519,
    IconBroom = 0xf51a,
    IconBrush = 0xf55d,
    IconBtc = 0xf15a,
    IconBuffer = 0xf837,
    IconBug = 0xf188,
    IconBuilding = 0xf1ad,
    IconBullhorn = 0xf0a1,
    IconBullseye = 0xf140,
    IconBurn = 0xf46a,
    IconBuromobelexperte = 0xf37f,
    IconBus = 0xf207,
    IconBusAlt = 0xf55e,
    IconBusinessTime = 0xf64a,
    IconBuyNLarge = 0xf8a6,
    IconBuysellads = 0xf20d,
    IconCalculator = 0xf1ec,
    IconCalendar = 0xf133,
    IconCalendarAlt = 0xf073,
    IconCalendarCheck = 0xf274,
    IconCalendarDay = 0xf783,
    IconCalendarMinus = 0xf272,
    IconCalendarPlus = 0xf271,
    IconCalendarTimes = 0xf273,
    IconCalendarWeek = 0xf784,
    IconCamera = 0xf030,
    IconCameraRetro = 0xf083,
    IconCampground = 0xf6bb,
    IconCanadianMapleLeaf = 0xf785,
    IconCandyCane = 0xf786,
    IconCannabis = 0xf55f,
    IconCapsules = 0xf46b,
    IconCar = 0xf1b9,
    IconCarAlt = 0xf5de,
    IconCarBattery = 0xf5df,
    IconCarCrash = 0xf5e1,
    IconCarSide = 0xf5e4,
    IconCaravan = 0xf8ff,
    IconCaretDown = 0xf0d7,
    IconCaretLeft = 0xf0d9,
    IconCaretRight = 0xf0da,
    IconCaretSquareDown = 0xf150,
    IconCaretSquareLeft = 0xf191,
    IconCaretSquareRight = 0xf152,
    IconCaretSquareUp = 0xf151,
    IconCaretUp = 0xf0d8,
    IconCarrot = 0xf787,
    IconCartArrowDown = 0xf218,
    IconCartPlus = 0xf217,
    IconCashRegister = 0xf788,
    IconCat = 0xf6be,
    IconCcAmazonPay = 0xf42d,
    IconCcAmex = 0xf1f3,
    IconCcApplePay = 0xf416,
    IconCcDinersClub = 0xf24c,
    IconCcDiscover = 0xf1f2,
    IconCcJcb = 0xf24b,
    IconCcMastercard = 0xf1f1,
    IconCcPaypal = 0xf1f4,
    IconCcStripe = 0xf1f5,
    IconCcVisa = 0xf1f0,
    IconCentercode = 0xf380,
    IconCentos = 0xf789,
    IconCertificate = 0xf0a3,
    IconChair = 0xf6c0,
    IconChalkboard = 0xf51b,
    IconChalkboardTeacher = 0xf51c,
    IconChargingStation = 0xf5e7,
    IconChartArea = 0xf1fe,
    IconChartBar = 0xf080,
    IconChartLine = 0xf201,
    IconChartPie = 0xf200,
    IconCheck = 0xf00c,
    IconCheckCircle = 0xf058,
    IconCheckDouble = 0xf560,
    IconCheckSquare = 0xf14a,
    IconCheese = 0xf7ef,
    IconChess = 0xf439,
    IconChessBishop = 0xf43a,
    IconChessBoard = 0xf43c,
    IconChessKing = 0xf43f,
    IconChessKnight = 0xf441,
    IconChessPawn = 0xf443,
    IconChessQueen = 0xf445,
    IconChessRook = 0xf447,
    IconChevronCircleDown = 0xf13a,
    IconChevronCircleLeft = 0xf137,
    IconChevronCircleRight = 0xf138,
    IconChevronCircleUp = 0xf139,
    IconChevronDown = 0xf078,
    IconChevronLeft = 0xf053,
    IconChevronRight = 0xf054,
    IconChevronUp = 0xf077,
    IconChild = 0xf1ae,
    IconChrome = 0xf268,
    IconChromecast = 0xf838,
    IconChurch = 0xf51d,
    IconCircle = 0xf111,
    IconCircleNotch = 0xf1ce,
    IconCity = 0xf64f,
    IconClinicMedical = 0xf7f2,
    IconClipboard = 0xf328,
    IconClipboardCheck = 0xf46c,
    IconClipboardList = 0xf46d,
    IconClock = 0xf017,
    IconClone = 0xf24d,
    IconClosedCaptioning = 0xf20a,
    IconCloud = 0xf0c2,
    IconCloudDownloadAlt = 0xf381,
    IconCloudMeatball = 0xf73b,
    IconCloudMoon = 0xf6c3,
    IconCloudMoonRain = 0xf73c,
    IconCloudRain = 0xf73d,
    IconCloudShowersHeavy = 0xf740,
    IconCloudSun = 0xf6c4,
    IconCloudSunRain = 0xf743,
    IconCloudUploadAlt = 0xf382,
    IconCloudflare = 0xe07d,
    IconCloudscale = 0xf383,
    IconCloudsmith = 0xf384,
    IconCloudversify = 0xf385,
    IconCocktail = 0xf561,
    IconCode = 0xf121,
    IconCodeBranch = 0xf126,
    IconCodepen = 0xf1cb,
    IconCodiepie = 0xf284,
    IconCoffee = 0xf0f4,
    IconCog = 0xf013,
    IconCogs = 0xf085,
    IconCoins = 0xf51e,
    IconColumns = 0xf0db,
    IconComment = 0xf075,
    IconCommentAlt = 0xf27a,
    IconCommentDollar = 0xf651,
    IconCommentDots = 0xf4ad,
    IconCommentMedical = 0xf7f5,
    IconCommentSlash = 0xf4b3,
    IconComments = 0xf086,
    IconCommentsDollar = 0xf653,
    IconCompactDisc = 0xf51f,
    IconCompass = 0xf14e,
    IconCompress = 0xf066,
    IconCompressAlt = 0xf422,
    IconCompressArrowsAlt = 0xf78c,
    IconConciergeBell = 0xf562,
    IconConfluence = 0xf78d,
    IconConnectdevelop = 0xf20e,
    IconContao = 0xf26d,
    IconCookie = 0xf563,
    IconCookieBite = 0xf564,
    IconCopy = 0xf0c5,
    IconCopyright = 0xf1f9,
    IconCottonBureau = 0xf89e,
    IconCouch = 0xf4b8,
    IconCpanel = 0xf388,
    IconCreativeCommons = 0xf25e,
    IconCreativeCommonsBy = 0xf4e7,
    IconCreativeCommonsNc = 0xf4e8,
    IconCreativeCommonsNcEu = 0xf4e9,
    IconCreativeCommonsNcJp = 0xf4ea,
    IconCreativeCommonsNd = 0xf4eb,
    IconCreativeCommonsPd = 0xf4ec,
    IconCreativeCommonsPdAlt = 0xf4ed,
    IconCreativeCommonsRemix = 0xf4ee,
    IconCreativeCommonsSa = 0xf4ef,
    IconCreativeCommonsSampling = 0xf4f0,
    IconCreativeCommonsSamplingPlus = 0xf4f1,
    IconCreativeCommonsShare = 0xf4f2,
    IconCreativeCommonsZero = 0xf4f3,
    IconCreditCard = 0xf09d,
    IconCriticalRole = 0xf6c9,
    IconCrop = 0xf125,
    IconCropAlt = 0xf565,
    IconCross = 0xf654,
    IconCrosshairs = 0xf05b,
    IconCrow = 0xf520,
    IconCrown = 0xf521,
    IconCrutch = 0xf7f7,
    IconCss3 = 0xf13c,
    IconCss3Alt = 0xf38b,
    IconCube = 0xf1b2,
    IconCubes = 0xf1b3,
    IconCut = 0xf0c4,
    IconCuttlefish = 0xf38c,
    IconDAndD = 0xf38d,
    IconDAndDBeyond = 0xf6ca,
    IconDailymotion = 0xe052,
    IconDashcube = 0xf210,
    IconDatabase = 0xf1c0,
    IconDeaf = 0xf2a4,
    IconDeezer = 0xe077,
    IconDelicious = 0xf1a5,
    IconDemocrat = 0xf747,
    IconDeploydog = 0xf38e,
    IconDeskpro = 0xf38f,
    IconDesktop = 0xf108,
    IconDev = 0xf6cc,
    IconDeviantart = 0xf1bd,
    IconDharmachakra = 0xf655,
    IconDhl = 0xf790,
    IconDiagnoses = 0xf470,
    IconDiaspora = 0xf791,
    IconDice = 0xf522,
    IconDiceD20 = 0xf6cf,
    IconDiceD6 = 0xf6d1,
    IconDiceFive = 0xf523,
    IconDiceFour = 0xf524,
    IconDiceOne = 0xf525,
    IconDiceSix = 0xf526,
    IconDiceThree = 0xf527,
    IconDiceTwo = 0xf528,
    IconDigg = 0xf1a6,
    IconDigitalOcean = 0xf391,
    IconDigitalTachograph = 0xf566,
    IconDirections = 0xf5eb,
    IconDiscord = 0xf392,
    IconDiscourse = 0xf393,
    IconDisease = 0xf7fa,
    IconDivide = 0xf529,
    IconDizzy = 0xf567,
    IconDna = 0xf471,
    IconDochub = 0xf394,
    IconDocker = 0xf395,
    IconDog = 0xf6d3,
    IconDollarSign = 0xf155,
    IconDolly = 0xf472,
    IconDollyFlatbed = 0xf474,
    IconDonate = 0xf4b9,
    IconDoorClosed = 0xf52a,
    IconDoorOpen = 0xf52b,
    IconDotCircle = 0xf192,
    IconDove = 0xf4ba,
    IconDownload = 0xf019,
    IconDraft2Digital = 0xf396,
    IconDraftingCompass = 0xf568,
    IconDragon = 0xf6d5,
    IconDrawPolygon = 0xf5ee,
    IconDribbble = 0xf17d,
    IconDribbbleSquare = 0xf397,
    IconDropbox = 0xf16b,
    IconDrum = 0xf569,
    IconDrumSteelpan = 0xf56a,
    IconDrumstickBite = 0xf6d7,
    IconDrupal = 0xf1a9,
    IconDumbbell = 0xf44b,
    IconDumpster = 0xf793,
    IconDumpsterFire = 0xf794,
    IconDungeon = 0xf6d9,
    IconDyalog = 0xf399,
    IconEarlybirds = 0xf39a,
    IconEbay = 0xf4f4,
    IconEdge = 0xf282,
    IconEdgeLegacy = 0xe078,
    IconEdit = 0xf044,
    IconEgg = 0xf7fb,
    IconEject = 0xf052,
    IconElementor = 0xf430,
    IconEllipsisH = 0xf141,
    IconEllipsisV = 0xf142,
    IconEllo = 0xf5f1,
    IconEmber = 0xf423,
    IconEmpire = 0xf1d1,
    IconEnvelope = 0xf0e0,
    IconEnvelopeOpen = 0xf2b6,
    IconEnvelopeOpenText = 0xf658,
    IconEnvelopeSquare = 0xf199,
    IconEnvira = 0xf299,
    IconEquals = 0xf52c,
    IconEraser = 0xf12d,
    IconErlang = 0xf39d,
    IconEthereum = 0xf42e,
    IconEthernet = 0xf796,
    IconEtsy = 0xf2d7,
    IconEuroSign = 0xf153,
    IconEvernote = 0xf839,
    IconExchangeAlt = 0xf362,
    IconExclamation = 0xf12a,
    IconExclamationCircle = 0xf06a,
    IconExclamationTriangle = 0xf071,
    IconExpand = 0xf065,
    IconExpandAlt = 0xf424,
    IconExpandArrowsAlt = 0xf31e,
    IconExpeditedssl = 0xf23e,
    IconExternalLinkAlt = 0xf35d,
    IconExternalLinkSquareAlt = 0xf360,
    IconEye = 0xf06e,
    IconEyeDropper = 0xf1fb,
    IconEyeSlash = 0xf070,
    IconFacebook = 0xf09a,
    IconFacebookF = 0xf39e,
    IconFacebookMessenger = 0xf39f,
    IconFacebookSquare = 0xf082,
    IconFan = 0xf863,
    IconFantasyFlightGames = 0xf6dc,
    IconFastBackward = 0xf049,
    IconFastForward = 0xf050,
    IconFaucet = 0xe005,
    IconFax = 0xf1ac,
    IconFeather = 0xf52d,
    IconFeatherAlt = 0xf56b,
    IconFedex = 0xf797,
    IconFedora = 0xf798,
    IconFemale = 0xf182,
    IconFighterJet = 0xf0fb,
    IconFigma = 0xf799,
    IconFile = 0xf15b,
    IconFileAlt = 0xf15c,
    IconFileArchive = 0xf1c6,
    IconFileAudio = 0xf1c7,
    IconFileCode = 0xf1c9,
    IconFileContract = 0xf56c,
    IconFileCsv = 0xf6dd,
    IconFileDownload = 0xf56d,
    IconFileExcel = 0xf1c3,
    IconFileExport = 0xf56e,
    IconFileImage = 0xf1c5,
    IconFileImport = 0xf56f,
    IconFileInvoice = 0xf570,
    IconFileInvoiceDollar = 0xf571,
    IconFileMedical = 0xf477,
    IconFileMedicalAlt = 0xf478,
    IconFilePdf = 0xf1c1,
    IconFilePowerpoint = 0xf1c4,
    IconFilePrescription = 0xf572,
    IconFileSignature = 0xf573,
    IconFileUpload = 0xf574,
    IconFileVideo = 0xf1c8,
    IconFileWord = 0xf1c2,
    IconFill = 0xf575,
    IconFillDrip = 0xf576,
    IconFilm = 0xf008,
    IconFilter = 0xf0b0,
    IconFingerprint = 0xf577,
    IconFire = 0xf06d,
    IconFireAlt = 0xf7e4,
    IconFireExtinguisher = 0xf134,
    IconFirefox = 0xf269,
    IconFirefoxBrowser = 0xe007,
    IconFirstAid = 0xf479,
    IconFirstOrder = 0xf2b0,
    IconFirstOrderAlt = 0xf50a,
    IconFirstdraft = 0xf3a1,
    IconFish = 0xf578,
    IconFistRaised = 0xf6de,
    IconFlag = 0xf024,
    IconFlagCheckered = 0xf11e,
    IconFlagUsa = 0xf74d,
    IconFlask = 0xf0c3,
    IconFlickr = 0xf16e,
    IconFlipboard = 0xf44d,
    IconFlushed = 0xf579,
    IconFly = 0xf417,
    IconFolder = 0xf07b,
    IconFolderMinus = 0xf65d,
    IconFolderOpen = 0xf07c,
    IconFolderPlus = 0xf65e,
    IconFont = 0xf031,
    IconFontAwesome = 0xf2b4,
    IconFontAwesomeAlt = 0xf35c,
    IconFontAwesomeFlag = 0xf425,
    IconFontAwesomeLogoFull = 0xf4e6,
    IconFonticons = 0xf280,
    IconFonticonsFi = 0xf3a2,
    IconFootballBall = 0xf44e,
    IconFortAwesome = 0xf286,
    IconFortAwesomeAlt = 0xf3a3,
    IconForumbee = 0xf211,
    IconForward = 0xf04e,
    IconFoursquare = 0xf180,
    IconFreeCodeCamp = 0xf2c5,
    IconFreebsd = 0xf3a4,
    IconFrog = 0xf52e,
    IconFrown = 0xf119,
    IconFrownOpen = 0xf57a,
    IconFulcrum = 0xf50b,
    IconFunnelDollar = 0xf662,
    IconFutbol = 0xf1e3,
    IconGalacticRepublic = 0xf50c,
    IconGalacticSenate = 0xf50d,
    IconGamepad = 0xf11b,
    IconGasPump = 0xf52f,
    IconGavel = 0xf0e3,
    IconGem = 0xf3a5,
    IconGenderless = 0xf22d,
    IconGetPocket = 0xf265,
    IconGg = 0xf260,
    IconGgCircle = 0xf261,
    IconGhost = 0xf6e2,
    IconGift = 0xf06b,
    IconGifts = 0xf79c,
    IconGit = 0xf1d3,
    IconGitAlt = 0xf841,
    IconGitSquare = 0xf1d2,
    IconGithub = 0xf09b,
    IconGithubAlt = 0xf113,
    IconGithubSquare = 0xf092,
    IconGitkraken = 0xf3a6,
    IconGitlab = 0xf296,
    IconGitter = 0xf426,
    IconGlassCheers = 0xf79f,
    IconGlassMartini = 0xf000,
    IconGlassMartiniAlt = 0xf57b,
    IconGlassWhiskey = 0xf7a0,
    IconGlasses = 0xf530,
    IconGlide = 0xf2a5,
    IconGlideG = 0xf2a6,
    IconGlobe = 0xf0ac,
    IconGlobeAfrica = 0xf57c,
    IconGlobeAmericas = 0xf57d,
    IconGlobeAsia = 0xf57e,
    IconGlobeEurope = 0xf7a2,
    IconGofore = 0xf3a7,
    IconGolfBall = 0xf450,
    IconGoodreads = 0xf3a8,
    IconGoodreadsG = 0xf3a9,
    IconGoogle = 0xf1a0,
    IconGoogleDrive = 0xf3aa,
    IconGooglePay = 0xe079,
    IconGooglePlay = 0xf3ab,
    IconGooglePlus = 0xf2b3,
    IconGooglePlusG = 0xf0d5,
    IconGooglePlusSquare = 0xf0d4,
    IconGoogleWallet = 0xf1ee,
    IconGopuram = 0xf664,
    IconGraduationCap = 0xf19d,
    IconGratipay = 0xf184,
    IconGrav = 0xf2d6,
    IconGreaterThan = 0xf531,
    IconGreaterThanEqual = 0xf532,
    IconGrimace = 0xf57f,
    IconGrin = 0xf580,
    IconGrinAlt = 0xf581,
    IconGrinBeam = 0xf582,
    IconGrinBeamSweat = 0xf583,
    IconGrinHearts = 0xf584,
    IconGrinSquint = 0xf585,
    IconGrinSquintTears = 0xf586,
    IconGrinStars = 0xf587,
    IconGrinTears = 0xf588,
    IconGrinTongue = 0xf589,
    IconGrinTongueSquint = 0xf58a,
    IconGrinTongueWink = 0xf58b,
    IconGrinWink = 0xf58c,
    IconGripHorizontal = 0xf58d,
    IconGripLines = 0xf7a4,
    IconGripLinesVertical = 0xf7a5,
    IconGripVertical = 0xf58e,
    IconGripfire = 0xf3ac,
    IconGrunt = 0xf3ad,
    IconGuilded = 0xe07e,
    IconGuitar = 0xf7a6,
    IconGulp = 0xf3ae,
    IconHSquare = 0xf0fd,
    IconHackerNews = 0xf1d4,
    IconHackerNewsSquare = 0xf3af,
    IconHackerrank = 0xf5f7,
    IconHamburger = 0xf805,
    IconHammer = 0xf6e3,
    IconHamsa = 0xf665,
    IconHandHolding = 0xf4bd,
    IconHandHoldingHeart = 0xf4be,
    IconHandHoldingMedical = 0xe05c,
    IconHandHoldingUsd = 0xf4c0,
    IconHandHoldingWater = 0xf4c1,
    IconHandLizard = 0xf258,
    IconHandMiddleFinger = 0xf806,
    IconHandPaper = 0xf256,
    IconHandPeace = 0xf25b,
    IconHandPointDown = 0xf0a7,
    IconHandPointLeft = 0xf0a5,
    IconHandPointRight = 0xf0a4,
    IconHandPointUp = 0xf0a6,
    IconHandPointer = 0xf25a,
    IconHandRock = 0xf255,
    IconHandScissors = 0xf257,
    IconHandSparkles = 0xe05d,
    IconHandSpock = 0xf259,
    IconHands = 0xf4c2,
    IconHandsHelping = 0xf4c4,
    IconHandsWash = 0xe05e,
    IconHandshake = 0xf2b5,
    IconHandshakeAltSlash = 0xe05f,
    IconHandshakeSlash = 0xe060,
    IconHanukiah = 0xf6e6,
    IconHardHat = 0xf807,
    IconHashtag = 0xf292,
    IconHatCowboy = 0xf8c0,
    IconHatCowboySide = 0xf8c1,
    IconHatWizard = 0xf6e8,
    IconHdd = 0xf0a0,
    IconHeadSideCough = 0xe061,
    IconHeadSideCoughSlash = 0xe062,
    IconHeadSideMask = 0xe063,
    IconHeadSideVirus = 0xe064,
    IconHeading = 0xf1dc,
    IconHeadphones = 0xf025,
    IconHeadphonesAlt = 0xf58f,
    IconHeadset = 0xf590,
    IconHeart = 0xf004,
    IconHeartBroken = 0xf7a9,
    IconHeartbeat = 0xf21e,
    IconHelicopter = 0xf533,
    IconHighlighter = 0xf591,
    IconHiking = 0xf6ec,
    IconHippo = 0xf6ed,
    IconHips = 0xf452,
    IconHireAHelper = 0xf3b0,
    IconHistory = 0xf1da,
    IconHive = 0xe07f,
    IconHockeyPuck = 0xf453,
    IconHollyBerry = 0xf7aa,
    IconHome = 0xf015,
    IconHooli = 0xf427,
    IconHornbill = 0xf592,
    IconHorse = 0xf6f0,
    IconHorseHead = 0xf7ab,
    IconHospital = 0xf0f8,
    IconHospitalAlt = 0xf47d,
    IconHospitalSymbol = 0xf47e,
    IconHospitalUser = 0xf80d,
    IconHotTub = 0xf593,
    IconHotdog = 0xf80f,
    IconHotel = 0xf594,
    IconHotjar = 0xf3b1,
    IconHourglass = 0xf254,
    IconHourglassEnd = 0xf253,
    IconHourglassHalf = 0xf252,
    IconHourglassStart = 0xf251,
    IconHouseDamage = 0xf6f1,
    IconHouseUser = 0xe065,
    IconHouzz = 0xf27c,
    IconHryvnia = 0xf6f2,
    IconHtml5 = 0xf13b,
    IconHubspot = 0xf3b2,
    IconICursor = 0xf246,
    IconIceCream = 0xf810,
    IconIcicles = 0xf7ad,
    IconIcons = 0xf86d,
    IconIdBadge = 0xf2c1,
    IconIdCard = 0xf2c2,
    IconIdCardAlt = 0xf47f,
    IconIdeal = 0xe013,
    IconIgloo = 0xf7ae,
    IconImage = 0xf03e,
    IconImages = 0xf302,
    IconImdb = 0xf2d8,
    IconInbox = 0xf01c,
    IconIndent = 0xf03c,
    IconIndustry = 0xf275,
    IconInfinity = 0xf534,
    IconInfo = 0xf129,
    IconInfoCircle = 0xf05a,
    IconInnosoft = 0xe080,
    IconInstagram = 0xf16d,
    IconInstagramSquare = 0xe055,
    IconInstalod = 0xe081,
    IconIntercom = 0xf7af,
    IconInternetExplorer = 0xf26b,
    IconInvision = 0xf7b0,
    IconIoxhost = 0xf208,
    IconItalic = 0xf033,
    IconItchIo = 0xf83a,
    IconItunes = 0xf3b4,
    IconItunesNote = 0xf3b5,
    IconJava = 0xf4e4,
    IconJedi = 0xf669,
    IconJediOrder = 0xf50e,
    IconJenkins = 0xf3b6,
    IconJira = 0xf7b1,
    IconJoget = 0xf3b7,
    IconJoint = 0xf595,
    IconJoomla = 0xf1aa,
    IconJournalWhills = 0xf66a,
    IconJs = 0xf3b8,
    IconJsSquare = 0xf3b9,
    IconJsfiddle = 0xf1cc,
    IconKaaba = 0xf66b,
    IconKaggle = 0xf5fa,
    IconKey = 0xf084,
    IconKeybase = 0xf4f5,
    IconKeyboard = 0xf11c,
    IconKeycdn = 0xf3ba,
    IconKhanda = 0xf66d,
    IconKickstarter = 0xf3bb,
    IconKickstarterK = 0xf3bc,
    IconKiss = 0xf596,
    IconKissBeam = 0xf597,
    IconKissWinkHeart = 0xf598,
    IconKiwiBird = 0xf535,
    IconKorvue = 0xf42f,
    IconLandmark = 0xf66f,
    IconLanguage = 0xf1ab,
    IconLaptop = 0xf109,
    IconLaptopCode = 0xf5fc,
    IconLaptopHouse = 0xe066,
    IconLaptopMedical = 0xf812,
    IconLaravel = 0xf3bd,
    IconLastfm = 0xf202,
    IconLastfmSquare = 0xf203,
    IconLaugh = 0xf599,
    IconLaughBeam = 0xf59a,
    IconLaughSquint = 0xf59b,
    IconLaughWink = 0xf59c,
    IconLayerGroup = 0xf5fd,
    IconLeaf = 0xf06c,
    IconLeanpub = 0xf212,
    IconLemon = 0xf094,
    IconLess = 0xf41d,
    IconLessThan = 0xf536,
    IconLessThanEqual = 0xf537,
    IconLevelDownAlt = 0xf3be,
    IconLevelUpAlt = 0xf3bf,
    IconLifeRing = 0xf1cd,
    IconLightbulb = 0xf0eb,
    IconLine = 0xf3c0,
    IconLink = 0xf0c1,
    IconLinkedin = 0xf08c,
    IconLinkedinIn = 0xf0e1,
    IconLinode = 0xf2b8,
    IconLinux = 0xf17c,
    IconLiraSign = 0xf195,
    IconList = 0xf03a,
    IconListAlt = 0xf022,
    IconListOl = 0xf0cb,
    IconListUl = 0xf0ca,
    IconLocationArrow = 0xf124,
    IconLock = 0xf023,
    IconLockOpen = 0xf3c1,
    IconLongArrowAltDown = 0xf309,
    IconLongArrowAltLeft = 0xf30a,
    IconLongArrowAltRight = 0xf30b,
    IconLongArrowAltUp = 0xf30c,
    IconLowVision = 0xf2a8,
    IconLuggageCart = 0xf59d,
    IconLungs = 0xf604,
    IconLungsVirus = 0xe067,
    IconLyft = 0xf3c3,
    IconMagento = 0xf3c4,
    IconMagic = 0xf0d0,
    IconMagnet = 0xf076,
    IconMailBulk = 0xf674,
    IconMailchimp = 0xf59e,
    IconMale = 0xf183,
    IconMandalorian = 0xf50f,
    IconMap = 0xf279,
    IconMapMarked = 0xf59f,
    IconMapMarkedAlt = 0xf5a0,
    IconMapMarker = 0xf041,
    IconMapMarkerAlt = 0xf3c5,
    IconMapPin = 0xf276,
    IconMapSigns = 0xf277,
    IconMarkdown = 0xf60f,
    IconMarker = 0xf5a1,
    IconMars = 0xf222,
    IconMarsDouble = 0xf227,
    IconMarsStroke = 0xf229,
    IconMarsStrokeH = 0xf22b,
    IconMarsStrokeV = 0xf22a,
    IconMask = 0xf6fa,
    IconMastodon = 0xf4f6,
    IconMaxcdn = 0xf136,
    IconMdb = 0xf8ca,
    IconMedal = 0xf5a2,
    IconMedapps = 0xf3c6,
    IconMedium = 0xf23a,
    IconMediumM = 0xf3c7,
    IconMedkit = 0xf0fa,
    IconMedrt = 0xf3c8,
    IconMeetup = 0xf2e0,
    IconMegaport = 0xf5a3,
    IconMeh = 0xf11a,
    IconMehBlank = 0xf5a4,
    IconMehRollingEyes = 0xf5a5,
    IconMemory = 0xf538,
    IconMendeley = 0xf7b3,
    IconMenorah = 0xf676,
    IconMercury = 0xf223,
    IconMeteor = 0xf753,
    IconMicroblog = 0xe01a,
    IconMicrochip = 0xf2db,
    IconMicrophone = 0xf130,
    IconMicrophoneAlt = 0xf3c9,
    IconMicrophoneAltSlash = 0xf539,
    IconMicrophoneSlash = 0xf131,
    IconMicroscope = 0xf610,
    IconMicrosoft = 0xf3ca,
    IconMinus = 0xf068,
    IconMinusCircle = 0xf056,
    IconMinusSquare = 0xf146,
    IconMitten = 0xf7b5,
    IconMix = 0xf3cb,
    IconMixcloud = 0xf289,
    IconMixer = 0xe056,
    IconMizuni = 0xf3cc,
    IconMobile = 0xf10b,
    IconMobileAlt = 0xf3cd,
    IconModx = 0xf285,
    IconMonero = 0xf3d0,
    IconMoneyBill = 0xf0d6,
    IconMoneyBillAlt = 0xf3d1,
    IconMoneyBillWave = 0xf53a,
    IconMoneyBillWaveAlt = 0xf53b,
    IconMoneyCheck = 0xf53c,
    IconMoneyCheckAlt = 0xf53d,
    IconMonument = 0xf5a6,
    IconMoon = 0xf186,
    IconMortarPestle = 0xf5a7,
    IconMosque = 0xf678,
    IconMotorcycle = 0xf21c,
    IconMountain = 0xf6fc,
    IconMouse = 0xf8cc,
    IconMousePointer = 0xf245,
    IconMugHot = 0xf7b6,
    IconMusic = 0xf001,
    IconNapster = 0xf3d2,
    IconNeos = 0xf612,
    IconNetworkWired = 0xf6ff,
    IconNeuter = 0xf22c,
    IconNewspaper = 0xf1ea,
    IconNimblr = 0xf5a8,
    IconNode = 0xf419,
    IconNodeJs = 0xf3d3,
    IconNotEqual = 0xf53e,
    IconNotesMedical = 0xf481,
    IconNpm = 0xf3d4,
    IconNs8 = 0xf3d5,
    IconNutritionix = 0xf3d6,
    IconObjectGroup = 0xf247,
    IconObjectUngroup = 0xf248,
    IconOctopusDeploy = 0xe082,
    IconOdnoklassniki = 0xf263,
    IconOdnoklassnikiSquare = 0xf264,
    IconOilCan = 0xf613,
    IconOldRepublic = 0xf510,
    IconOm = 0xf679,
    IconOpencart = 0xf23d,
    IconOpenid = 0xf19b,
    IconOpera = 0xf26a,
    IconOptinMonster = 0xf23c,
    IconOrcid = 0xf8d2,
    IconOsi = 0xf41a,
    IconOtter = 0xf700,
    IconOutdent = 0xf03b,
    IconPage4 = 0xf3d7,
    IconPagelines = 0xf18c,
    IconPager = 0xf815,
    IconPaintBrush = 0xf1fc,
    IconPaintRoller = 0xf5aa,
    IconPalette = 0xf53f,
    IconPalfed = 0xf3d8,
    IconPallet = 0xf482,
    IconPaperPlane = 0xf1d8,
    IconPaperclip = 0xf0c6,
    IconParachuteBox = 0xf4cd,
    IconParagraph = 0xf1dd,
    IconParking = 0xf540,
    IconPassport = 0xf5ab,
    IconPastafarianism = 0xf67b,
    IconPaste = 0xf0ea,
    IconPatreon = 0xf3d9,
    IconPause = 0xf04c,
    IconPauseCircle = 0xf28b,
    IconPaw = 0xf1b0,
    IconPaypal = 0xf1ed,
    IconPeace = 0xf67c,
    IconPen = 0xf304,
    IconPenAlt = 0xf305,
    IconPenFancy = 0xf5ac,
    IconPenNib = 0xf5ad,
    IconPenSquare = 0xf14b,
    IconPencilAlt = 0xf303,
    IconPencilRuler = 0xf5ae,
    IconPennyArcade = 0xf704,
    IconPeopleArrows = 0xe068,
    IconPeopleCarry = 0xf4ce,
    IconPepperHot = 0xf816,
    IconPerbyte = 0xe083,
    IconPercent = 0xf295,
    IconPercentage = 0xf541,
    IconPeriscope = 0xf3da,
    IconPersonBooth = 0xf756,
    IconPhabricator = 0xf3db,
    IconPhoenixFramework = 0xf3dc,
    IconPhoenixSquadron = 0xf511,
    IconPhone = 0xf095,
    IconPhoneAlt = 0xf879,
    IconPhoneSlash = 0xf3dd,
    IconPhoneSquare = 0xf098,
    IconPhoneSquareAlt = 0xf87b,
    IconPhoneVolume = 0xf2a0,
    IconPhotoVideo = 0xf87c,
    IconPhp = 0xf457,
    IconPiedPiper = 0xf2ae,
    IconPiedPiperAlt = 0xf1a8,
    IconPiedPiperHat = 0xf4e5,
    IconPiedPiperPp = 0xf1a7,
    IconPiedPiperSquare = 0xe01e,
    IconPiggyBank = 0xf4d3,
    IconPills = 0xf484,
    IconPinterest = 0xf0d2,
    IconPinterestP = 0xf231,
    IconPinterestSquare = 0xf0d3,
    IconPizzaSlice = 0xf818,
    IconPlaceOfWorship = 0xf67f,
    IconPlane = 0xf072,
    IconPlaneArrival = 0xf5af,
    IconPlaneDeparture = 0xf5b0,
    IconPlaneSlash = 0xe069,
    IconPlay = 0xf04b,
    IconPlayCircle = 0xf144,
    IconPlaystation = 0xf3df,
    IconPlug = 0xf1e6,
    IconPlus = 0xf067,
    IconPlusCircle = 0xf055,
    IconPlusSquare = 0xf0fe,
    IconPodcast = 0xf2ce,
    IconPoll = 0xf681,
    IconPollH = 0xf682,
    IconPoo = 0xf2fe,
    IconPooStorm = 0xf75a,
    IconPoop = 0xf619,
    IconPortrait = 0xf3e0,
    IconPoundSign = 0xf154,
    IconPowerOff = 0xf011,
    IconPray = 0xf683,
    IconPrayingHands = 0xf684,
    IconPrescription = 0xf5b1,
    IconPrescriptionBottle = 0xf485,
    IconPrescriptionBottleAlt = 0xf486,
    IconPrint = 0xf02f,
    IconProcedures = 0xf487,
    IconProductHunt = 0xf288,
    IconProjectDiagram = 0xf542,
    IconPumpMedical = 0xe06a,
    IconPumpSoap = 0xe06b,
    IconPushed = 0xf3e1,
    IconPuzzlePiece = 0xf12e,
    IconPython = 0xf3e2,
    IconQq = 0xf1d6,
    IconQrcode = 0xf029,
    IconQuestion = 0xf128,
    IconQuestionCircle = 0xf059,
    IconQuidditch = 0xf458,
    IconQuinscape = 0xf459,
    IconQuora = 0xf2c4,
    IconQuoteLeft = 0xf10d,
    IconQuoteRight = 0xf10e,
    IconQuran = 0xf687,
    IconRProject = 0xf4f7,
    IconRadiation = 0xf7b9,
    IconRadiationAlt = 0xf7ba,
    IconRainbow = 0xf75b,
    IconRandom = 0xf074,
    IconRaspberryPi = 0xf7bb,
    IconRavelry = 0xf2d9,
    IconReact = 0xf41b,
    IconReacteurope = 0xf75d,
    IconReadme = 0xf4d5,
    IconRebel = 0xf1d0,
    IconReceipt = 0xf543,
    IconRecordVinyl = 0xf8d9,
    IconRecycle = 0xf1b8,
    IconRedRiver = 0xf3e3,
    IconReddit = 0xf1a1,
    IconRedditAlien = 0xf281,
    IconRedditSquare = 0xf1a2,
    IconRedhat = 0xf7bc,
    IconRedo = 0xf01e,
    IconRedoAlt = 0xf2f9,
    IconRegistered = 0xf25d,
    IconRemoveFormat = 0xf87d,
    IconRenren = 0xf18b,
    IconReply = 0xf3e5,
    IconReplyAll = 0xf122,
    IconReplyd = 0xf3e6,
    IconRepublican = 0xf75e,
    IconResearchgate = 0xf4f8,
    IconResolving = 0xf3e7,
    IconRestroom = 0xf7bd,
    IconRetweet = 0xf079,
    IconRev = 0xf5b2,
    IconRibbon = 0xf4d6,
    IconRing = 0xf70b,
    IconRoad = 0xf018,
    IconRobot = 0xf544,
    IconRocket = 0xf135,
    IconRocketchat = 0xf3e8,
    IconRockrms = 0xf3e9,
    IconRoute = 0xf4d7,
    IconRss = 0xf09e,
    IconRssSquare = 0xf143,
    IconRubleSign = 0xf158,
    IconRuler = 0xf545,
    IconRulerCombined = 0xf546,
    IconRulerHorizontal = 0xf547,
    IconRulerVertical = 0xf548,
    IconRunning = 0xf70c,
    IconRupeeSign = 0xf156,
    IconRust = 0xe07a,
    IconSadCry = 0xf5b3,
    IconSadTear = 0xf5b4,
    IconSafari = 0xf267,
    IconSalesforce = 0xf83b,
    IconSass = 0xf41e,
    IconSatellite = 0xf7bf,
    IconSatelliteDish = 0xf7c0,
    IconSave = 0xf0c7,
    IconSchlix = 0xf3ea,
    IconSchool = 0xf549,
    IconScrewdriver = 0xf54a,
    IconScribd = 0xf28a,
    IconScroll = 0xf70e,
    IconSdCard = 0xf7c2,
    IconSearch = 0xf002,
    IconSearchDollar = 0xf688,
    IconSearchLocation = 0xf689,
    IconSearchMinus = 0xf010,
    IconSearchPlus = 0xf00e,
    IconSearchengin = 0xf3eb,
    IconSeedling = 0xf4d8,
    IconSellcast = 0xf2da,
    IconSellsy = 0xf213,
    IconServer = 0xf233,
    IconServicestack = 0xf3ec,
    IconShapes = 0xf61f,
    IconShare = 0xf064,
    IconShareAlt = 0xf1e0,
    IconShareAltSquare = 0xf1e1,
    IconShareSquare = 0xf14d,
    IconShekelSign = 0xf20b,
    IconShieldAlt = 0xf3ed,
    IconShieldVirus = 0xe06c,
    IconShip = 0xf21a,
    IconShippingFast = 0xf48b,
    IconShirtsinbulk = 0xf214,
    IconShoePrints = 0xf54b,
    IconShopify = 0xe057,
    IconShoppingBag = 0xf290,
    IconShoppingBasket = 0xf291,
    IconShoppingCart = 0xf07a,
    IconShopware = 0xf5b5,
    IconShower = 0xf2cc,
    IconShuttleVan = 0xf5b6,
    IconSign = 0xf4d9,
    IconSignInAlt = 0xf2f6,
    IconSignLanguage = 0xf2a7,
    IconSignOutAlt = 0xf2f5,
    IconSignal = 0xf012,
    IconSignature = 0xf5b7,
    IconSimCard = 0xf7c4,
    IconSimplybuilt = 0xf215,
    IconSink = 0xe06d,
    IconSistrix = 0xf3ee,
    IconSitemap = 0xf0e8,
    IconSith = 0xf512,
    IconSkating = 0xf7c5,
    IconSketch = 0xf7c6,
    IconSkiing = 0xf7c9,
    IconSkiingNordic = 0xf7ca,
    IconSkull = 0xf54c,
    IconSkullCrossbones = 0xf714,
    IconSkyatlas = 0xf216,
    IconSkype = 0xf17e,
    IconSlack = 0xf198,
    IconSlackHash = 0xf3ef,
    IconSlash = 0xf715,
    IconSleigh = 0xf7cc,
    IconSlidersH = 0xf1de,
    IconSlideshare = 0xf1e7,
    IconSmile = 0xf118,
    IconSmileBeam = 0xf5b8,
    IconSmileWink = 0xf4da,
    IconSmog = 0xf75f,
    IconSmoking = 0xf48d,
    IconSmokingBan = 0xf54d,
    IconSms = 0xf7cd,
    IconSnapchat = 0xf2ab,
    IconSnapchatGhost = 0xf2ac,
    IconSnapchatSquare = 0xf2ad,
    IconSnowboarding = 0xf7ce,
    IconSnowflake = 0xf2dc,
    IconSnowman = 0xf7d0,
    IconSnowplow = 0xf7d2,
    IconSoap = 0xe06e,
    IconSocks = 0xf696,
    IconSolarPanel = 0xf5ba,
    IconSort = 0xf0dc,
    IconSortAlphaDown = 0xf15d,
    IconSortAlphaDownAlt = 0xf881,
    IconSortAlphaUp = 0xf15e,
    IconSortAlphaUpAlt = 0xf882,
    IconSortAmountDown = 0xf160,
    IconSortAmountDownAlt = 0xf884,
    IconSortAmountUp = 0xf161,
    IconSortAmountUpAlt = 0xf885,
    IconSortDown = 0xf0dd,
    IconSortNumericDown = 0xf162,
    IconSortNumericDownAlt = 0xf886,
    IconSortNumericUp = 0xf163,
    IconSortNumericUpAlt = 0xf887,
    IconSortUp = 0xf0de,
    IconSoundcloud = 0xf1be,
    IconSourcetree = 0xf7d3,
    IconSpa = 0xf5bb,
    IconSpaceShuttle = 0xf197,
    IconSpeakap = 0xf3f3,
    IconSpeakerDeck = 0xf83c,
    IconSpellCheck = 0xf891,
    IconSpider = 0xf717,
    IconSpinner = 0xf110,
    IconSplotch = 0xf5bc,
    IconSpotify = 0xf1bc,
    IconSprayCan = 0xf5bd,
    IconSquare = 0xf0c8,
    IconSquareFull = 0xf45c,
    IconSquareRootAlt = 0xf698,
    IconSquarespace = 0xf5be,
    IconStackExchange = 0xf18d,
    IconStackOverflow = 0xf16c,
    IconStackpath = 0xf842,
    IconStamp = 0xf5bf,
    IconStar = 0xf005,
    IconStarAndCrescent = 0xf699,
    IconStarHalf = 0xf089,
    IconStarHalfAlt = 0xf5c0,
    IconStarOfDavid = 0xf69a,
    IconStarOfLife = 0xf621,
    IconStaylinked = 0xf3f5,
    IconSteam = 0xf1b6,
    IconSteamSquare = 0xf1b7,
    IconSteamSymbol = 0xf3f6,
    IconStepBackward = 0xf048,
    IconStepForward = 0xf051,
    IconStethoscope = 0xf0f1,
    IconStickerMule = 0xf3f7,
    IconStickyNote = 0xf249,
    IconStop = 0xf04d,
    IconStopCircle = 0xf28d,
    IconStopwatch = 0xf2f2,
    IconStopwatch20 = 0xe06f,
    IconStore = 0xf54e,
    IconStoreAlt = 0xf54f,
    IconStoreAltSlash = 0xe070,
    IconStoreSlash = 0xe071,
    IconStrava = 0xf428,
    IconStream = 0xf550,
    IconStreetView = 0xf21d,
    IconStrikethrough = 0xf0cc,
    IconStripe = 0xf429,
    IconStripeS = 0xf42a,
    IconStroopwafel = 0xf551,
    IconStudiovinari = 0xf3f8,
    IconStumbleupon = 0xf1a4,
    IconStumbleuponCircle = 0xf1a3,
    IconSubscript = 0xf12c,
    IconSubway = 0xf239,
    IconSuitcase = 0xf0f2,
    IconSuitcaseRolling = 0xf5c1,
    IconSun = 0xf185,
    IconSuperpowers = 0xf2dd,
    IconSuperscript = 0xf12b,
    IconSupple = 0xf3f9,
    IconSurprise = 0xf5c2,
    IconSuse = 0xf7d6,
    IconSwatchbook = 0xf5c3,
    IconSwift = 0xf8e1,
    IconSwimmer = 0xf5c4,
    IconSwimmingPool = 0xf5c5,
    IconSymfony = 0xf83d,
    IconSynagogue = 0xf69b,
    IconSync = 0xf021,
    IconSyncAlt = 0xf2f1,
    IconSyringe = 0xf48e,
    IconTable = 0xf0ce,
    IconTableTennis = 0xf45d,
    IconTablet = 0xf10a,
    IconTabletAlt = 0xf3fa,
    IconTablets = 0xf490,
    IconTachometerAlt = 0xf3fd,
    IconTag = 0xf02b,
    IconTags = 0xf02c,
    IconTape = 0xf4db,
    IconTasks = 0xf0ae,
    IconTaxi = 0xf1ba,
    IconTeamspeak = 0xf4f9,
    IconTeeth = 0xf62e,
    IconTeethOpen = 0xf62f,
    IconTelegram = 0xf2c6,
    IconTelegramPlane = 0xf3fe,
    IconTemperatureHigh = 0xf769,
    IconTemperatureLow = 0xf76b,
    IconTencentWeibo = 0xf1d5,
    IconTenge = 0xf7d7,
    IconTerminal = 0xf120,
    IconTextHeight = 0xf034,
    IconTextWidth = 0xf035,
    IconTh = 0xf00a,
    IconThLarge = 0xf009,
    IconThList = 0xf00b,
    IconTheRedYeti = 0xf69d,
    IconTheaterMasks = 0xf630,
    IconThemeco = 0xf5c6,
    IconThemeisle = 0xf2b2,
    IconThermometer = 0xf491,
    IconThermometerEmpty = 0xf2cb,
    IconThermometerFull = 0xf2c7,
    IconThermometerHalf = 0xf2c9,
    IconThermometerQuarter = 0xf2ca,
    IconThermometerThreeQuarters = 0xf2c8,
    IconThinkPeaks = 0xf731,
    IconThumbsDown = 0xf165,
    IconThumbsUp = 0xf164,
    IconThumbtack = 0xf08d,
    IconTicketAlt = 0xf3ff,
    IconTiktok = 0xe07b,
    IconTimes = 0xf00d,
    IconTimesCircle = 0xf057,
    IconTint = 0xf043,
    IconTintSlash = 0xf5c7,
    IconTired = 0xf5c8,
    IconToggleOff = 0xf204,
    IconToggleOn = 0xf205,
    IconToilet = 0xf7d8,
    IconToiletPaper = 0xf71e,
    IconToiletPaperSlash = 0xe072,
    IconToolbox = 0xf552,
    IconTools = 0xf7d9,
    IconTooth = 0xf5c9,
    IconTorah = 0xf6a0,
    IconToriiGate = 0xf6a1,
    IconTractor = 0xf722,
    IconTradeFederation = 0xf513,
    IconTrademark = 0xf25c,
    IconTrafficLight = 0xf637,
    IconTrailer = 0xe041,
    IconTrain = 0xf238,
    IconTram = 0xf7da,
    IconTransgender = 0xf224,
    IconTransgenderAlt = 0xf225,
    IconTrash = 0xf1f8,
    IconTrashAlt = 0xf2ed,
    IconTrashRestore = 0xf829,
    IconTrashRestoreAlt = 0xf82a,
    IconTree = 0xf1bb,
    IconTrello = 0xf181,
    IconTripadvisor = 0xf262,
    IconTrophy = 0xf091,
    IconTruck = 0xf0d1,
    IconTruckLoading = 0xf4de,
    IconTruckMonster = 0xf63b,
    IconTruckMoving = 0xf4df,
    IconTruckPickup = 0xf63c,
    IconTshirt = 0xf553,
    IconTty = 0xf1e4,
    IconTumblr = 0xf173,
    IconTumblrSquare = 0xf174,
    IconTv = 0xf26c,
    IconTwitch = 0xf1e8,
    IconTwitter = 0xf099,
    IconTwitterSquare = 0xf081,
    IconTypo3 = 0xf42b,
    IconUber = 0xf402,
    IconUbuntu = 0xf7df,
    IconUikit = 0xf403,
    IconUmbraco = 0xf8e8,
    IconUmbrella = 0xf0e9,
    IconUmbrellaBeach = 0xf5ca,
    IconUncharted = 0xe084,
    IconUnderline = 0xf0cd,
    IconUndo = 0xf0e2,
    IconUndoAlt = 0xf2ea,
    IconUniregistry = 0xf404,
    IconUnity = 0xe049,
    IconUniversalAccess = 0xf29a,
    IconUniversity = 0xf19c,
    IconUnlink = 0xf127,
    IconUnlock = 0xf09c,
    IconUnlockAlt = 0xf13e,
    IconUnsplash = 0xe07c,
    IconUntappd = 0xf405,
    IconUpload = 0xf093,
    IconUps = 0xf7e0,
    IconUsb = 0xf287,
    IconUser = 0xf007,
    IconUserAlt = 0xf406,
    IconUserAltSlash = 0xf4fa,
    IconUserAstronaut = 0xf4fb,
    IconUserCheck = 0xf4fc,
    IconUserCircle = 0xf2bd,
    IconUserClock = 0xf4fd,
    IconUserCog = 0xf4fe,
    IconUserEdit = 0xf4ff,
    IconUserFriends = 0xf500,
    IconUserGraduate = 0xf501,
    IconUserInjured = 0xf728,
    IconUserLock = 0xf502,
    IconUserMd = 0xf0f0,
    IconUserMinus = 0xf503,
    IconUserNinja = 0xf504,
    IconUserNurse = 0xf82f,
    IconUserPlus = 0xf234,
    IconUserSecret = 0xf21b,
    IconUserShield = 0xf505,
    IconUserSlash = 0xf506,
    IconUserTag = 0xf507,
    IconUserTie = 0xf508,
    IconUserTimes = 0xf235,
    IconUsers = 0xf0c0,
    IconUsersCog = 0xf509,
    IconUsersSlash = 0xe073,
    IconUsps = 0xf7e1,
    IconUssunnah = 0xf407,
    IconUtensilSpoon = 0xf2e5,
    IconUtensils = 0xf2e7,
    IconVaadin = 0xf408,
    IconVectorSquare = 0xf5cb,
    IconVenus = 0xf221,
    IconVenusDouble = 0xf226,
    IconVenusMars = 0xf228,
    IconVest = 0xe085,
    IconVestPatches = 0xe086,
    IconViacoin = 0xf237,
    IconViadeo = 0xf2a9,
    IconViadeoSquare = 0xf2aa,
    IconVial = 0xf492,
    IconVials = 0xf493,
    IconViber = 0xf409,
    IconVideo = 0xf03d,
    IconVideoSlash = 0xf4e2,
    IconVihara = 0xf6a7,
    IconVimeo = 0xf40a,
    IconVimeoSquare = 0xf194,
    IconVimeoV = 0xf27d,
    IconVine = 0xf1ca,
    IconVirus = 0xe074,
    IconVirusSlash = 0xe075,
    IconViruses = 0xe076,
    IconVk = 0xf189,
    IconVnv = 0xf40b,
    IconVoicemail = 0xf897,
    IconVolleyballBall = 0xf45f,
    IconVolumeDown = 0xf027,
    IconVolumeMute = 0xf6a9,
    IconVolumeOff = 0xf026,
    IconVolumeUp = 0xf028,
    IconVoteYea = 0xf772,
    IconVrCardboard = 0xf729,
    IconVuejs = 0xf41f,
    IconWalking = 0xf554,
    IconWallet = 0xf555,
    IconWarehouse = 0xf494,
    IconWatchmanMonitoring = 0xe087,
    IconWater = 0xf773,
    IconWaveSquare = 0xf83e,
    IconWaze = 0xf83f,
    IconWeebly = 0xf5cc,
    IconWeibo = 0xf18a,
    IconWeight = 0xf496,
    IconWeightHanging = 0xf5cd,
    IconWeixin = 0xf1d7,
    IconWhatsapp = 0xf232,
    IconWhatsappSquare = 0xf40c,
    IconWheelchair = 0xf193,
    IconWhmcs = 0xf40d,
    IconWifi = 0xf1eb,
    IconWikipediaW = 0xf266,
    IconWind = 0xf72e,
    IconWindowClose = 0xf410,
    IconWindowMaximize = 0xf2d0,
    IconWindowMinimize = 0xf2d1,
    IconWindowRestore = 0xf2d2,
    IconWindows = 0xf17a,
    IconWineBottle = 0xf72f,
    IconWineGlass = 0xf4e3,
    IconWineGlassAlt = 0xf5ce,
    IconWix = 0xf5cf,
    IconWizardsOfTheCoast = 0xf730,
    IconWodu = 0xe088,
    IconWolfPackBattalion = 0xf514,
    IconWonSign = 0xf159,
    IconWordpress = 0xf19a,
    IconWordpressSimple = 0xf411,
    IconWpbeginner = 0xf297,
    IconWpexplorer = 0xf2de,
    IconWpforms = 0xf298,
    IconWpressr = 0xf3e4,
    IconWrench = 0xf0ad,
    IconXRay = 0xf497,
    IconXbox = 0xf412,
    IconXing = 0xf168,
    IconXingSquare = 0xf169,
    IconYCombinator = 0xf23b,
    IconYahoo = 0xf19e,
    IconYammer = 0xf840,
    IconYandex = 0xf413,
    IconYandexInternational = 0xf414,
    IconYarn = 0xf7e3,
    IconYelp = 0xf1e9,
    IconYenSign = 0xf157,
    IconYinYang = 0xf6ad,
    IconYoast = 0xf2b1,
    IconYoutube = 0xf167,
    IconYoutubeSquare = 0xf431,
    IconZhihu = 0xf63f,
};

#endif // ICONS_H
