/*
    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>

    This file is part of CopyQ.

    CopyQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CopyQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CopyQ.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MIMETYPES_H
#define MIMETYPES_H

#define COPYQ_MIME_PREFIX "application/x-copyq-"
extern const char mimeText[];
extern const char mimeHtml[];
extern const char mimeUriList[];
extern const char mimeWindowTitle[];
extern const char mimeItems[];
extern const char mimeItemNotes[];
extern const char mimeIcon[];
extern const char mimeOwner[];
extern const char mimeClipboardMode[];
extern const char mimeCurrentTab[];
extern const char mimeSelectedItems[];
extern const char mimeCurrentItem[];
extern const char mimeHidden[];
extern const char mimeShortcut[];
extern const char mimeColor[];
extern const char mimeOutputTab[];

#endif // MIMETYPES_H
